@@ -38,82 +38,89 @@ namespace gensios {
 	    Event *cb = g->get_cb();
 	    Gensio *g2;
 
+	    if (!cb)
+		return GE_NOTSUP;
+
 	    try {
 		if (event >= GENSIO_EVENT_USER_MIN &&
-		    event <= GENSIO_EVENT_USER_MAX) {
+				event <= GENSIO_EVENT_USER_MAX) {
 		    std::vector<unsigned char> val(buf, buf + *buflen);
 		    return cb->user_event(event, err, val, auxdata);
 		}
 
 		if (event >= SERGENSIO_EVENT_BASE &&
-		    event <= SERGENSIO_EVENT_MAX) {
-		    Serial_Event *scb = dynamic_cast<Serial_Event *>(cb);
+				event <= SERGENSIO_EVENT_MAX) {
 		    unsigned int *val = (unsigned int *) buf;
 
-		    if (!scb)
-			return GE_NOTSUP;
-
 		    if (event == GENSIO_EVENT_SER_SIGNATURE) {
 			std::vector<unsigned char> sig(buf, buf + *buflen);
-			scb->signature(std::move(sig));
+			cb->signature(std::move(sig));
 			return 0;
 		    }
 
 		    switch (event) {
 		    case GENSIO_EVENT_SER_MODEMSTATE:
-			scb->modemstate(*val);
+			cb->modemstate(*val);
+			break;
+
+		    case GENSIO_EVENT_SER_MODEMSTATE_MASK:
+			cb->modemstate_mask(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_LINESTATE:
-			scb->linestate(*val);
+			cb->linestate(*val);
+			break;
+
+		    case GENSIO_EVENT_SER_LINESTATE_MASK:
+			cb->linestate_mask(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_FLOW_STATE:
-			scb->flow_state(*val);
+			cb->flow_state(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_FLUSH:
-			scb->flush(*val);
+			cb->flush(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_SYNC:
-			scb->sync();
+			cb->sync();
 			break;
 
 		    case GENSIO_EVENT_SER_BAUD:
-			scb->baud(*val);
+			cb->baud(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_DATASIZE:
-			scb->datasize(*val);
+			cb->datasize(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_PARITY:
-			scb->parity(*val);
+			cb->parity(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_STOPBITS:
-			scb->stopbits(*val);
+			cb->stopbits(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_FLOWCONTROL:
-			scb->flowcontrol(*val);
+			cb->flowcontrol(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_IFLOWCONTROL:
-			scb->iflowcontrol(*val);
+			cb->iflowcontrol(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_SBREAK:
-			scb->sbreak(*val);
+			cb->sbreak(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_DTR:
-			scb->dtr(*val);
+			cb->dtr(*val);
 			break;
 
 		    case GENSIO_EVENT_SER_RTS:
-			scb->rts(*val);
+			cb->rts(*val);
 			break;
 
 		    default:
@@ -210,7 +217,7 @@ namespace gensios {
 		    std::string outstr(len + 1, '\0');
 		    vsnprintf(&outstr[0], len + 1, d->log, d->args);
 		    cb->parmlog(outstr);
-		    break;
+		    return 0;
 		}
 
 		case GENSIO_EVENT_WIN_SIZE: {
@@ -218,7 +225,23 @@ namespace gensios {
 
 		    sscanf((char *) buf, "%u:%u", &height, &width);
 		    cb->win_size(height, width);
-		    break;
+		    return 0;
+		}
+
+		case GENSIO_EVENT_LOG: {
+		    struct gensio_log_data *d =
+			(struct gensio_log_data *) buf;
+		    va_list argcopy;
+		    va_copy(argcopy, d->args);
+		    size_t len = vsnprintf(NULL, 0, d->log, argcopy);
+		    va_end(argcopy);
+		    std::string outstr(len + 1, '\0');
+		    // Copy args again, in case the user doesn't handle them
+		    // and returns GE_NOTSUP.
+		    va_copy(argcopy, d->args);
+		    vsnprintf(&outstr[0], len + 1, d->log, argcopy);
+		    va_end(argcopy);
+		    return cb->log(d->level, outstr);
 		}
 		}
 		return GE_NOTSUP;
@@ -359,20 +382,37 @@ namespace gensios {
 				  const char *const *auxdata)
     {
 	Event *cb = static_cast<Event *>(user_data);
-	struct gensio_parmlog_data *p;
 
-	if (event != GENSIO_EVENT_PARMLOG)
+	switch(event) {
+	case GENSIO_EVENT_PARMLOG: {
+	    struct gensio_parmlog_data *p = (struct gensio_parmlog_data *) buf;
+
+	    va_list argcopy;
+	    va_copy(argcopy, p->args);
+	    size_t len = vsnprintf(NULL, 0, p->log, argcopy);
+	    va_end(argcopy);
+	    std::string outstr(len, '\0');
+	    vsnprintf(&outstr[0], len + 1, p->log, p->args);
+	    cb->parmlog(outstr);
+	    return 0;
+	}
+
+	case GENSIO_EVENT_LOG: {
+	    struct gensio_log_data *p = (struct gensio_log_data *) buf;
+
+	    va_list argcopy;
+	    va_copy(argcopy, p->args);
+	    size_t len = vsnprintf(NULL, 0, p->log, argcopy);
+	    va_end(argcopy);
+	    std::string outstr(len, '\0');
+	    vsnprintf(&outstr[0], len + 1, p->log, p->args);
+	    cb->log(p->level, outstr);
+	    return 0;
+	}
+
+	default:
 	    return GE_NOTSUP;
-
-	p = (struct gensio_parmlog_data *) buf;
-	va_list argcopy;
-	va_copy(argcopy, p->args);
-	size_t len = vsnprintf(NULL, 0, p->log, argcopy);
-	va_end(argcopy);
-	std::string outstr(len, '\0');
-	vsnprintf(&outstr[0], len + 1, p->log, p->args);
-	cb->parmlog(outstr);
-	return 0;
+	}
     }
 
     Gensio *
@@ -596,30 +636,75 @@ namespace gensios {
 	return gensio_control(io, depth, get, option, data, datalen);
     }
 
+    static void gensio_cpp_control_done(struct gensio *io, int err,
+					const char *buf, gensiods len,
+					void *user_data)
+    {
+	if (!user_data)
+	    return;
+	struct gensio_frdata *f = gensio_get_frdata(io);
+	struct gensio_cpp_data *d = gensio_container_of(f,
+					      struct gensio_cpp_data, frdata);
+	Gensio *g = d->g;
+	Gensio_Control_Done *done
+	    = static_cast<Gensio_Control_Done *>(user_data);
+	std::vector<unsigned char> valv(buf, buf + len);
+
+	try {
+	    done->control_done(err, std::move(valv));
+	} catch (std::exception &e) {
+	    gensio_log(g->get_os_funcs(), GENSIO_LOG_ERR,
+		       "Received C++ exception in control done handler: %s",
+		       e.what());
+	}
+    }
+
+    int Gensio::acontrol(int depth, bool get, unsigned int option,
+			 const char *data, gensiods datalen,
+			 Gensio_Control_Done *done,
+			 gensio_time *timeout)
+    {
+	int err;
+
+	if (done)
+	    err = gensio_acontrol(io, depth, get, option, data, datalen,
+				  gensio_cpp_control_done, done,
+				  timeout);
+	else
+	    err = gensio_acontrol(io, depth, get, option, data, datalen,
+				  NULL, NULL, timeout);
+	return err;
+    }
+
+    int Gensio::acontrol_s(int depth, bool get, unsigned int option,
+			   char *data, gensiods *datalen,
+			   gensio_time *timeout, bool intr)
+    {
+	if (intr)
+	    return gensio_acontrol_s_intr(io, depth, get, option, data, datalen,
+					  timeout);
+	else
+	    return gensio_acontrol_s(io, depth, get, option, data, datalen,
+				     timeout);
+    }
+
     int Gensio::read_s(std::vector<unsigned char> &rvec,
 		       gensio_time *timeout, bool intr)
     {
 	int err;
 	gensiods len = (gensiods) rvec.capacity(), count = 0;
-	unsigned char *buf;
+	unsigned char *buf = rvec.data();
 
-	buf = (unsigned char *) go->zalloc(go, len);
-	if (!buf)
-	    throw gensio_error(GE_NOMEM);
+	rvec.resize(len);
 	if (intr)
 	    err = gensio_read_s_intr(io, &count, buf, len, timeout);
 	else
 	    err = gensio_read_s(io, &count, buf, len, timeout);
-	if (err == GE_TIMEDOUT || err== GE_INTERRUPTED) {
-	    go->free(go, (void *) buf);
+	if (err == GE_TIMEDOUT || err== GE_INTERRUPTED)
 	    return err;
-	}
-	if (err) {
-	    go->free(go, (void *) buf);
+	if (err)
 	    throw gensio_error(err);
-	}
-	rvec.assign(buf, buf + count);
-	go->free(go, (void *) buf);
+	rvec.resize(count);
 	return 0;
     }
 
