@@ -10,6 +10,7 @@ import (
 	"fmt"
 	"github.com/cminyard/go/gensio"
 	"testbase"
+	"runtime"
 )
 
 type FreeDone struct {
@@ -40,16 +41,36 @@ type WatchEvent struct {
 func (we *WatchEvent) Event(state int, interfacenum int, ipdomain int,
 		name string, mtype string, domain string, host string,
 		addr gensio.Addr, txt []string) {
-	if state == gensio.MDNS_ALL_FOR_NOW {
-		we.w.Wake()
-	} else if state == gensio.MDNS_NEW_DATA {
-		we.found = true
-		we.w.Wake()
+	if state == gensio.MDNS_WATCH_ALL_FOR_NOW {
+		/* Don't do a wake here, it's not reliable. */
+	} else if state == gensio.MDNS_WATCH_NEW_DATA {
+		if !we.found {
+			we.found = true
+			we.w.Wake()
+		}
 	} else {
 		we.watchCount++
 	}
 }
 
+type ServiceEvent struct {
+	gensio.MDNSServiceEventBase
+	w *gensio.Waiter
+}
+
+func (se *ServiceEvent) Event(ev int, info string) {
+	if ev == gensio.MDNS_SERVICE_READY {
+		fmt.Printf("Service ready: %s\n", info);
+	} else if ev == gensio.MDNS_SERVICE_READY_NEW_NAME {
+		fmt.Printf("Service ready with new name: %s\n", info);
+	} else if ev == gensio.MDNS_SERVICE_REMOVED {
+		fmt.Printf("Service removed\n");
+		se.w.Wake()
+	} else {
+		fmt.Printf("Error from service: %s\n", info);
+	}
+}
+
 func main() {
 	fmt.Println("Starting MDNS Go tests")
 
@@ -65,11 +86,21 @@ func main() {
 	we := &WatchEvent{}
 	we.w = waiter
 	testbase.ObjCount++
-	w := m.AddWatch(-1, gensio.GENSIO_NETTYPE_UNSPEC, "=gensio2",
-		"@_gensio2.*", nil, nil, we)
+	var namestr string
+	if runtime.GOOS == "windows" {
+		// No glob support in windows.
+		namestr = "gensio2"
+	} else {
+		namestr = "@gen*"
+	}
+	w := m.AddWatch(-1, gensio.GENSIO_NETTYPE_UNSPEC, namestr,
+		"_gensio2._tcp", nil, nil, we)
 	testbase.ObjCount++
+	se := &ServiceEvent{}
+	se.w = waiter
 	s := m.AddService(-1, gensio.GENSIO_NETTYPE_UNSPEC, "gensio2",
-		"_gensio2._tcp", nil, nil, 5001, []string{"A", "B"})
+		"_gensio2._tcp", nil, nil, 5001, []string{"A=1", "B=2"},
+	        se)
 	testbase.ObjCount++
 
 	rv := waiter.Wait(1, gensio.NewTime(5, 0))
@@ -93,6 +124,10 @@ func main() {
 	w = nil
 
 	s.Free()
+	rv = waiter.Wait(1, gensio.NewTime(5, 0))
+	if rv != 0 {
+		panic("Error waiting for service free: " + gensio.ErrToStr(rv))
+	}
 	s = nil
 
 	testbase.ObjCount++
@@ -107,6 +142,7 @@ func main() {
 	fd = nil
 	m = nil
 	we = nil
+	se = nil
 
 	waiter = nil
 	o = nil
