@@ -60,12 +60,136 @@ extern "C" {
 #define GENSIO_EVENT_REQUEST_2FA	10
 #define GENSIO_EVENT_2FA_VERIFY		11
 
+struct gensio_parmlog_data {
+    const char *log;
+    va_list args;
+};
+#define GENSIO_EVENT_PARMLOG		12
+#define GENSIO_EVENT_WIN_SIZE		13
+
+struct gensio_log_data {
+    enum gensio_log_levels level;
+    const char *log;
+    va_list args;
+};
+#define GENSIO_EVENT_LOG		14
+
 /*
  * Serial callbacks start here and run to 2000.
  */
 #define SERGENSIO_EVENT_BASE	1000
 #define SERGENSIO_EVENT_MAX	1999
 
+/*
+ * Events for dynamic changes to the serial port.  Users can ignore these
+ * if they don't care.
+ */
+
+/*
+ * Client side only, these are for reporting changes to the user.  buf
+ * points to an unsigned integer holding the modem or line state.
+ */
+#define GENSIO_EVENT_SER_MODEMSTATE	(SERGENSIO_EVENT_BASE + 1)
+#define GENSIO_EVENT_SER_LINESTATE	(SERGENSIO_EVENT_BASE + 2)
+
+/*
+ * On the server side, these are for reporting that the client is
+ * requesting the signature.  Not for client.
+ */
+#define GENSIO_EVENT_SER_SIGNATURE	(SERGENSIO_EVENT_BASE + 3)
+
+/*
+ * The remote end is asking the user to flow control or flush.  Client
+ * or server.
+ */
+#define GENSIO_EVENT_SER_FLOW_STATE	(SERGENSIO_EVENT_BASE + 4)
+#define GENSIO_EVENT_SER_FLUSH		(SERGENSIO_EVENT_BASE + 5)
+
+/* Got a sync from the other end.  Client or server. */
+#define GENSIO_EVENT_SER_SYNC		(SERGENSIO_EVENT_BASE + 6)
+
+/*
+ * Server callbacks.  These only come in in server mode, you must
+ * call the equivalent sergensio_xxx() function to return the response,
+ * though the done callback is ignored in that case.  buf points to
+ * an integer holding the value.
+ */
+
+/* Baud, datasize, and stopbits use numbers. */
+#define GENSIO_EVENT_SER_BAUD		(SERGENSIO_EVENT_BASE + 7)
+#define GENSIO_EVENT_SER_DATASIZE	(SERGENSIO_EVENT_BASE + 8)
+
+#define GENSIO_EVENT_SER_PARITY		(SERGENSIO_EVENT_BASE + 9)
+
+#define GENSIO_SER_PARITY_NONE	1u
+#define GENSIO_SER_PARITY_ODD	2u
+#define GENSIO_SER_PARITY_EVEN	3u
+#define GENSIO_SER_PARITY_MARK	4u
+#define GENSIO_SER_PARITY_SPACE	5u
+GENSIO_DLL_PUBLIC
+const char *gensio_parity_to_str(unsigned int ival);
+GENSIO_DLL_PUBLIC
+int gensio_str_to_parity(const char *sval);
+
+
+#define GENSIO_EVENT_SER_STOPBITS	(SERGENSIO_EVENT_BASE + 10)
+#define GENSIO_EVENT_SER_FLOWCONTROL	(SERGENSIO_EVENT_BASE + 11)
+#define GENSIO_EVENT_SER_IFLOWCONTROL	(SERGENSIO_EVENT_BASE + 12)
+
+#define GENSIO_SER_FLOWCONTROL_NONE	1u
+#define GENSIO_SER_FLOWCONTROL_XON_XOFF	2u
+#define GENSIO_SER_FLOWCONTROL_RTS_CTS	3u
+#define GENSIO_SER_FLOWCONTROL_DCD	4u
+#define GENSIO_SER_FLOWCONTROL_DTR	5u
+#define GENSIO_SER_FLOWCONTROL_DSR	6u
+GENSIO_DLL_PUBLIC
+const char *gensio_flowcontrol_to_str(unsigned int ival);
+GENSIO_DLL_PUBLIC
+int gensio_str_to_flowcontrol(const char *sval);
+
+/* This can be used for all the "on" and "off" event values below. */
+#define GENSIO_SER_ON	1u
+#define GENSIO_SER_OFF	2u
+GENSIO_DLL_PUBLIC
+const char *gensio_onoff_to_str(unsigned int ival);
+GENSIO_DLL_PUBLIC
+int gensio_str_to_onoff(const char *sval);
+
+#define GENSIO_EVENT_SER_SBREAK		(SERGENSIO_EVENT_BASE + 13)
+#define GENSIO_EVENT_SER_DTR		(SERGENSIO_EVENT_BASE + 14)
+#define GENSIO_EVENT_SER_RTS		(SERGENSIO_EVENT_BASE + 15)
+
+/*
+ * On the server side, this is for reporting that the client has
+ * requested the mask be changed.  buf points to an unsigned integer
+ * holding the new modem or line state mask.
+ */
+#define GENSIO_EVENT_SER_MODEMSTATE_MASK (SERGENSIO_EVENT_BASE + 16)
+#define GENSIO_EVENT_SER_LINESTATE_MASK	(SERGENSIO_EVENT_BASE + 17)
+
+/*
+ * For linestate and modemstate, on a client this sets the mask, on
+ * the server this is reporting the current state to the client.
+ */
+#define GENSIO_SER_LINESTATE_DATA_READY		(1 << 0)
+#define GENSIO_SER_LINESTATE_OVERRUN_ERR	(1 << 1)
+#define GENSIO_SER_LINESTATE_PARITY_ERR		(1 << 2)
+#define GENSIO_SER_LINESTATE_FRAMING_ERR	(1 << 3)
+#define GENSIO_SER_LINESTATE_BREAK		(1 << 4)
+#define GENSIO_SER_LINESTATE_XMIT_HOLD_EMPTY	(1 << 5)
+#define GENSIO_SER_LINESTATE_XMIT_SHIFT_EMPTY	(1 << 6)
+#define GENSIO_SER_LINESTATE_TIMEOUT_ERR	(1 << 7)
+
+/* Note that for modemstate you should use the low 4 bits. */
+#define GENSIO_SER_MODEMSTATE_CTS_CHANGED	(1 << 0)
+#define GENSIO_SER_MODEMSTATE_DSR_CHANGED	(1 << 1)
+#define GENSIO_SER_MODEMSTATE_RI_CHANGED	(1 << 2)
+#define GENSIO_SER_MODEMSTATE_CD_CHANGED	(1 << 3)
+#define GENSIO_SER_MODEMSTATE_CTS		(1 << 4)
+#define GENSIO_SER_MODEMSTATE_DSR		(1 << 5)
+#define GENSIO_SER_MODEMSTATE_RI		(1 << 6)
+#define GENSIO_SER_MODEMSTATE_CD		(1 << 7)
+
 /*
  * If a user creates their own gensio with their own events, they should
  * use this range.
@@ -177,6 +301,23 @@ GENSIO_DLL_PUBLIC
 int gensio_control(struct gensio *io, int depth, bool get,
 		   unsigned int option, char *data, gensiods *datalen);
 
+GENSIO_DLL_PUBLIC
+int gensio_acontrol(struct gensio *io, int depth, bool get,
+		    unsigned int option, const char *data,
+		    gensiods datalen,
+		    gensio_control_done done, void *cb_data,
+		    gensio_time *timeout);
+
+GENSIO_DLL_PUBLIC
+int gensio_acontrol_s(struct gensio *io, int depth, bool get,
+		      unsigned int option, char *data,
+		      gensiods *datalen, gensio_time *timeout);
+
+GENSIO_DLL_PUBLIC
+int gensio_acontrol_s_intr(struct gensio *io, int depth, bool get,
+			   unsigned int option, char *data,
+			   gensiods *datalen, gensio_time *timeout);
+
 #include <gensio/gensio_control.h>
 
 GENSIO_DLL_PUBLIC
@@ -195,6 +336,10 @@ GENSIO_DLL_PUBLIC
 bool gensio_is_encrypted(struct gensio *io);
 GENSIO_DLL_PUBLIC
 bool gensio_is_message(struct gensio *io);
+GENSIO_DLL_PUBLIC
+bool gensio_is_mux(struct gensio *io);
+GENSIO_DLL_PUBLIC
+bool gensio_is_serial(struct gensio *io);
 
 GENSIO_DLL_PUBLIC
 int gensio_set_sync(struct gensio *io);
@@ -253,7 +398,7 @@ struct gensio_accepter *gensio_acc_get_child(struct gensio_accepter *acc,
 #define GENSIO_ACC_EVENT_LOG		2
 struct gensio_loginfo {
     enum gensio_log_levels level;
-    char *str;
+    const char *str;
     va_list args;
 };
 
@@ -279,6 +424,8 @@ struct gensio_acc_postcert_verify_data {
 #define GENSIO_ACC_EVENT_REQUEST_2FA		9
 /* Uses struct gensio_acc_password_verify_data */
 
+#define GENSIO_ACC_EVENT_PARMLOG		10
+
 GENSIO_DLL_PUBLIC
 int str_to_gensio_accepter(const char *str, struct gensio_os_funcs *o,
 			   gensio_accepter_event cb, void *user_data,
@@ -384,6 +531,10 @@ GENSIO_DLL_PUBLIC
 bool gensio_acc_is_packet(struct gensio_accepter *accepter);
 GENSIO_DLL_PUBLIC
 bool gensio_acc_is_message(struct gensio_accepter *accepter);
+GENSIO_DLL_PUBLIC
+bool gensio_acc_is_mux(struct gensio_accepter *accepter);
+GENSIO_DLL_PUBLIC
+bool gensio_acc_is_serial(struct gensio_accepter *accepter);
 
 /*
  * This is data for frameworks that sit on top of gensio, added to do
@@ -413,11 +564,6 @@ enum gensio_default_type {
     GENSIO_DEFAULT_DATA
 };
 
-struct gensio_enum_val {
-    char *name;
-    int val;
-};
-
 GENSIO_DLL_PUBLIC
 int gensio_add_default(struct gensio_os_funcs *o,
 		       const char *name,
