@@ -17,6 +17,9 @@
 #define LOCK(l) AcquireSRWLockExclusive(l)
 #define UNLOCK(l) ReleaseSRWLockExclusive(l)
 #define LOCK_INITIALIZER SRWLOCK_INIT
+#define lock_thread_type DWORD
+#define LOCK_GET_THREAD_ID GetCurrentThreadId()
+#define LOCK_THREAD_ID_EQUAL(a,b) ((a) == (b))
 #elif defined(USE_PTHREADS)
 #include <pthread.h>
 #define lock_type pthread_mutex_t
@@ -25,6 +28,9 @@
 #define LOCK(l) pthread_mutex_lock(l)
 #define UNLOCK(l) pthread_mutex_unlock(l)
 #define LOCK_INITIALIZER PTHREAD_MUTEX_INITIALIZER
+#define lock_thread_type pthread_t
+#define LOCK_GET_THREAD_ID pthread_self()
+#define LOCK_THREAD_ID_EQUAL(a,b) pthread_equal(a,b)
 #else
 #include <assert.h>
 #define lock_type int
@@ -33,4 +39,7 @@
 #define LOCK(l) do { assert(*l == 0); *l = 1; } while(0)
 #define UNLOCK(l) do { assert(*l == 1); *l = 0; } while(0)
 #define LOCK_INITIALIZER 0
+#define lock_thread_type int
+#define LOCK_GET_THREAD_ID 0
+#define LOCK_THREAD_ID_EQUAL(a,b) ((a) == (b))
 #endif
