@@ -96,6 +96,7 @@ struct gensio {
     bool is_encrypted;
     bool is_message;
     bool is_mux;
+    bool is_serial;
 
     struct gensio_sync_io *sync_io;
 
@@ -128,10 +129,12 @@ gensio_base_init(void *cb_data)
     struct gensio_os_funcs *o = cb_data;
 
     gensio_base_lock = o->alloc_lock(o);
-    if (!gensio_base_lock)
+    if (!gensio_base_lock) {
 	gensio_base_init_rv = GE_NOMEM;
-    else
+    } else {
+	o->get_funcs(o);
 	o_base = o;
+    }
 }
 
 struct gensio *
@@ -259,6 +262,34 @@ gensio_cb(struct gensio *io, int event, int err,
     return rv;
 }
 
+void
+gensio_gvlog(struct gensio *io, enum gensio_log_levels level,
+	     const char *log, va_list args)
+{
+    struct gensio_log_data data;
+    int rv;
+
+    data.level = level;
+    data.log = log;
+    va_copy(data.args, args);
+    rv = gensio_cb(io, GENSIO_EVENT_LOG, 0, (unsigned char *) &data,
+		   NULL, NULL);
+    if (rv == GE_NOTSUP)
+	gensio_vlog(io->o, level, log, args);
+    va_end(data.args);
+}
+
+void
+gensio_glog(struct gensio *io, enum gensio_log_levels level,
+	    const char *log, ...)
+{
+    va_list args;
+
+    va_start(args, log);
+    gensio_gvlog(io, level, log, args);
+    va_end(args);
+}
+
 int
 gensio_addclass(struct gensio *io, const char *name, int classops_ver,
 		struct gensio_classops *ops, void *classdata)
@@ -343,6 +374,7 @@ struct gensio_accepter {
     bool is_reliable;
     bool is_message;
     bool is_mux;
+    bool is_serial;
     bool sync;
     bool enabled;
 
@@ -744,6 +776,133 @@ gensio_control(struct gensio *io, int depth, bool get,
     return c->func(c, GENSIO_FUNC_CONTROL, datalen, &get, option, data, NULL);
 }
 
+int
+gensio_acontrol(struct gensio *io, int depth, bool get,
+		unsigned int option, const char *data,
+		gensiods datalen,
+		gensio_control_done done, void *cb_data,
+		gensio_time *timeout)
+{
+    struct gensio_func_acontrol ctrldata = {
+	.data = data,
+	.datalen = datalen,
+	.timeout = timeout,
+	.done = done,
+	.cb_data = cb_data
+    };
+    struct gensio *c = io;
+
+    if (depth == GENSIO_CONTROL_DEPTH_FIRST) {
+	while (c) {
+	    int rv = c->func(c, GENSIO_FUNC_ACONTROL, NULL, &get, option,
+			     &ctrldata, NULL);
+
+	    if (rv != GE_NOTSUP)
+		return rv;
+	    c = c->child;
+	}
+	return GE_NOTFOUND;
+    }
+
+    if (depth < 0)
+	return GE_INVAL;
+
+    while (depth > 0) {
+	if (!c->child)
+	    return GE_NOTFOUND;
+	depth--;
+	c = c->child;
+    }
+
+    return c->func(c, GENSIO_FUNC_ACONTROL, NULL, &get, option,
+		   &ctrldata, NULL);
+}
+
+struct gensio_acontrol_s_data {
+    bool is_get;
+    struct gensio_os_funcs *o;
+    struct gensio_waiter *waiter;
+    char *data;
+    gensiods datalen;
+    int err;
+};
+
+static void
+gensio_acontrol_s_done(struct gensio *io, int err, const char *str,
+		       gensiods len, void *cb_data)
+{
+    struct gensio_acontrol_s_data *data = cb_data;
+    gensiods copysize = len;
+
+    data->err = err;
+    if (!err && data->is_get) {
+	if (copysize > data->datalen - 1)
+	    copysize = data->datalen - 1;
+	if (copysize)
+	    memcpy(data->data, str, len);
+	data->datalen = len;
+    }
+    data->o->wake(data->waiter);
+}
+
+int
+gensio_acontrol_s(struct gensio *io, int depth, bool get,
+		  unsigned int option, char *idata,
+		  gensiods *datalen, gensio_time *timeout)
+{
+    struct gensio_os_funcs *o = io->o;
+    struct gensio_acontrol_s_data data = {
+	.is_get = get,
+	.o = o,
+	.waiter = o->alloc_waiter(o),
+	.data = idata,
+	.datalen = *datalen,
+	.err = 0
+    };
+    int rv;
+
+    if (!data.waiter)
+	return GE_NOMEM;
+
+    rv = gensio_acontrol(io, depth, get, option, idata, *datalen,
+			 gensio_acontrol_s_done, &data, timeout);
+    if (rv)
+	return rv;
+    o->wait(data.waiter, 1, NULL);
+    o->free_waiter(data.waiter);
+    *datalen = data.datalen;
+    return data.err;
+}
+
+int
+gensio_acontrol_s_intr(struct gensio *io, int depth, bool get,
+		       unsigned int option, char *idata,
+		       gensiods *datalen, gensio_time *timeout)
+{
+    struct gensio_os_funcs *o = io->o;
+    struct gensio_acontrol_s_data data = {
+	.is_get = get,
+	.o = o,
+	.waiter = o->alloc_waiter(o),
+	.data = idata,
+	.datalen = *datalen,
+	.err = 0
+    };
+    int rv;
+
+    if (!data.waiter)
+	return GE_NOMEM;
+
+    rv = gensio_acontrol(io, depth, get, option, idata, *datalen,
+			 gensio_acontrol_s_done, &data, timeout);
+    if (rv)
+	return rv;
+    rv = o->wait_intr(data.waiter, 1, timeout);
+    o->free_waiter(data.waiter);
+    *datalen = data.datalen;
+    return data.err;
+}
+
 const char *
 gensio_get_type(struct gensio *io, unsigned int depth)
 {
@@ -907,6 +1066,12 @@ gensio_is_mux(struct gensio *io)
     return io->is_mux;
 }
 
+bool
+gensio_is_serial(struct gensio *io)
+{
+    return io->is_serial;
+}
+
 void
 gensio_set_is_client(struct gensio *io, bool is_client)
 {
@@ -949,6 +1114,12 @@ gensio_set_is_mux(struct gensio *io, bool is_mux)
     io->is_mux = is_mux;
 }
 
+void
+gensio_set_is_serial(struct gensio *io, bool is_serial)
+{
+    io->is_serial = is_serial;
+}
+
 void
 gensio_set_frdata(struct gensio *io, struct gensio_frdata *frdata)
 {
@@ -969,6 +1140,7 @@ gensio_set_attr_from_child(struct gensio *io, struct gensio *child)
     gensio_set_is_authenticated(io, gensio_is_authenticated(child));
     gensio_set_is_encrypted(io, gensio_is_encrypted(child));
     gensio_set_is_message(io, gensio_is_message(child));
+    gensio_set_is_serial(io, gensio_is_serial(child));
 }
 
 struct gensio_accepter *
@@ -1246,6 +1418,12 @@ gensio_acc_is_mux(struct gensio_accepter *accepter)
     return accepter->is_mux;
 }
 
+bool
+gensio_acc_is_serial(struct gensio_accepter *accepter)
+{
+    return accepter->is_serial;
+}
+
 void
 gensio_acc_set_is_reliable(struct gensio_accepter *accepter, bool is_reliable)
 {
@@ -1270,6 +1448,12 @@ gensio_acc_set_is_mux(struct gensio_accepter *accepter, bool is_mux)
      accepter->is_mux = is_mux;
 }
 
+void
+gensio_acc_set_is_serial(struct gensio_accepter *accepter, bool is_serial)
+{
+     accepter->is_serial = is_serial;
+}
+
 struct registered_gensio_accepter {
     const char *name;
     str_to_gensio_acc_handler handler;
@@ -1391,7 +1575,9 @@ gensio_loadlib(struct gensio_os_funcs *o, const char *str)
     memcpy(name, str, len);
     name[len] = '\0';
     if (strcmp(name, "tcp") == 0 || strcmp(name, "unix") == 0)
-	strcpy(name, "net");
+	strncpy(name, "net", sizeof(name));
+    if (strcmp(name, "dev") == 0 || strcmp(name, "sdev") == 0)
+	strncpy(name, "serialdev", sizeof(name));
 
     return gensio_os_loadlib(o, name);
 }
@@ -2393,7 +2579,7 @@ gensio_pparm_unknown_parm(struct gensio_pparm_info *p,
 
 void
 gensio_acc_vlog(struct gensio_accepter *acc, enum gensio_log_levels level,
-		char *str, va_list args)
+		const char *str, va_list args)
 {
     struct gensio_loginfo info;
 
@@ -2409,7 +2595,7 @@ gensio_acc_vlog(struct gensio_accepter *acc, enum gensio_log_levels level,
 
 void
 gensio_acc_log(struct gensio_accepter *acc, enum gensio_log_levels level,
-	       char *str, ...)
+	       const char *str, ...)
 {
     va_list args;
 
@@ -2421,6 +2607,7 @@ gensio_acc_log(struct gensio_accepter *acc, enum gensio_log_levels level,
 static struct gensio_once gensio_default_initialized;
 
 static struct gensio_lock *deflock;
+static struct gensio_os_funcs *deflock_osh;
 
 struct gensio_def_val {
     char *strval;
@@ -2493,10 +2680,11 @@ struct gensio_def_entry builtin_defaults[] = {
     { "xonxoff",	GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
     { "rtscts",		GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
     { "local",		GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
-    { "hangup_when_done", GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
+    { "hangup-when-done", GENSIO_DEFAULT_INT,	.def.intval = -1 },
     { "custspeed",	GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
     { "rs485",		GENSIO_DEFAULT_STR,	.def.strval = NULL },
-    { "nouucplock",	GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
+    { "uucplock",	GENSIO_DEFAULT_BOOL,	.def.intval = 1 },
+    { "flock",		GENSIO_DEFAULT_BOOL,	.def.intval = 1 },
     { "drain_time",	GENSIO_DEFAULT_INT,	.min = -1, .max = INT_MAX,
 						.def.intval = -1, },
     { "char_drain_wait",GENSIO_DEFAULT_INT,	.min = -1, .max = INT_MAX,
@@ -2555,6 +2743,8 @@ struct gensio_def_entry builtin_defaults[] = {
 						.def.intval = -1 },
     { "nettype",	GENSIO_DEFAULT_STR,	.def.strval = "unspec" },
     { "nostack",	GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
+    { "mdnstimeout",	GENSIO_DEFAULT_INT,	.def.intval = 1000 },
+    { "ignore-v6-link-local",GENSIO_DEFAULT_BOOL,.def.intval = 0 },
     { NULL }
 };
 
@@ -2571,12 +2761,14 @@ gensio_default_init(void *cb_data)
     struct gensio_os_funcs *o = cb_data;
 
     deflock = o->alloc_lock(o);
-    if (!deflock)
+    if (!deflock) {
 	gensio_def_init_rv = GE_NOMEM;
-    else
+    } else {
+	deflock_osh = o->get_funcs(o);
 	/* Default reuseaddr to false for UDP. */
 	gensio_def_init_rv = l_gensio_set_default(o, "udp", "reuseaddr",
 						  NULL, 0);
+    }
 }
 
 void
@@ -2607,6 +2799,9 @@ gensio_cleanup_mem(struct gensio_os_funcs *o)
     if (deflock)
 	o->free_lock(deflock);
     deflock = NULL;
+    if (deflock_osh)
+	deflock_osh->free_funcs(deflock_osh);
+    deflock_osh = NULL;
 
     if (reg_gensio_acc_lock)
 	o->free_lock(reg_gensio_acc_lock);
@@ -2645,6 +2840,11 @@ gensio_cleanup_mem(struct gensio_os_funcs *o)
 	o->free_lock(cleanups_lock);
     cleanups_lock = NULL;
 
+    if (o_base) {
+	o_base->free_funcs(o_base);
+	o_base = NULL;
+    }
+
     reg_o = NULL;
 }
 
@@ -2676,12 +2876,12 @@ l_gensio_reset_defaults(struct gensio_os_funcs *o)
     unsigned int i;
 
     if (deflock) {
-	o->lock(deflock);
+	deflock_osh->lock(deflock);
 	for (i = 0; builtin_defaults[i].name; i++)
 	    gensio_reset_default(o, &builtin_defaults[i]);
 	for (d = defaults; d; d = d->next)
 	    gensio_reset_default(o, d);
-	o->unlock(deflock);
+	deflock_osh->unlock(deflock);
     }
 }
 
@@ -2758,7 +2958,7 @@ gensio_add_default(struct gensio_os_funcs *o,
     if (gensio_def_init_rv)
 	return gensio_def_init_rv;
 
-    o->lock(deflock);
+    deflock_osh->lock(deflock);
     d = gensio_lookup_default(name, NULL, NULL);
     if (d) {
 	err = GE_EXISTS;
@@ -2804,7 +3004,7 @@ gensio_add_default(struct gensio_os_funcs *o,
     defaults = d;
 
  out_unlock:
-    o->unlock(deflock);
+    deflock_osh->unlock(deflock);
     return err;
 }
 
@@ -2818,7 +3018,7 @@ l_gensio_set_default(struct gensio_os_funcs *o,
     char *new_strval = NULL, *end;
     unsigned int i;
 
-    o->lock(deflock);
+    deflock_osh->lock(deflock);
     d = gensio_lookup_default(name, NULL, NULL);
     if (!d) {
 	err = GE_NOTFOUND;
@@ -2945,7 +3145,7 @@ l_gensio_set_default(struct gensio_os_funcs *o,
  out_unlock:
     if (new_strval)
 	o->free(o, new_strval);
-    o->unlock(deflock);
+    deflock_osh->unlock(deflock);
     return err;
 }
 
@@ -2977,7 +3177,7 @@ gensio_get_default(struct gensio_os_funcs *o,
     if (gensio_def_init_rv)
 	return gensio_def_init_rv;
 
-    o->lock(deflock);
+    deflock_osh->lock(deflock);
     d = gensio_lookup_default(name, NULL, NULL);
     if (!d) {
 	err = GE_NOTFOUND;
@@ -3045,7 +3245,7 @@ gensio_get_default(struct gensio_os_funcs *o,
     }
 
  out_unlock:
-    o->unlock(deflock);
+    deflock_osh->unlock(deflock);
 
     return err;
 }
@@ -3063,7 +3263,7 @@ gensio_del_default(struct gensio_os_funcs *o,
     if (gensio_def_init_rv)
 	return gensio_def_init_rv;
 
-    o->lock(deflock);
+    deflock_osh->lock(deflock);
     d = gensio_lookup_default(name, &prev, &isdefault);
     if (!d) {
 	err = GE_NOTFOUND;
@@ -3118,7 +3318,7 @@ gensio_del_default(struct gensio_os_funcs *o,
     o->free(o, d);
 
  out_unlock:
-    o->unlock(deflock);
+    deflock_osh->unlock(deflock);
 
     return err;
 }
@@ -3745,5 +3945,124 @@ gensio_time_diff_nsecs(gensio_time *t1, gensio_time *t2)
     return v;
 }
 
+const char *
+gensio_parity_to_str(unsigned int ival)
+{
+    switch(ival) {
+    case 0:
+	return "0";
+
+    case GENSIO_SER_PARITY_NONE:
+	return "none";
+
+    case GENSIO_SER_PARITY_ODD:
+	return "odd";
+
+    case GENSIO_SER_PARITY_EVEN:
+	return "even";
+
+    case GENSIO_SER_PARITY_MARK:
+	return "mark";
+
+    case GENSIO_SER_PARITY_SPACE:
+	return "space";
+
+    default:
+	return "?";
+    }
+}
+
+int
+gensio_str_to_parity(const char *sval)
+{
+    if (strcmp(sval, "none") == 0)
+	return GENSIO_SER_PARITY_NONE;
+    if (strcmp(sval, "odd") == 0)
+	return GENSIO_SER_PARITY_ODD;
+    if (strcmp(sval, "even") == 0)
+	return GENSIO_SER_PARITY_EVEN;
+    if (strcmp(sval, "mark") == 0)
+	return GENSIO_SER_PARITY_MARK;
+    if (strcmp(sval, "space") == 0)
+	return GENSIO_SER_PARITY_SPACE;
+    return -1;
+}
+
+const char *
+gensio_flowcontrol_to_str(unsigned int ival)
+{
+    switch (ival) {
+    case 0:
+	return "0";
+
+    case GENSIO_SER_FLOWCONTROL_NONE:
+	return "none";
+
+    case GENSIO_SER_FLOWCONTROL_XON_XOFF:
+	return "xonxoff";
+
+    case GENSIO_SER_FLOWCONTROL_RTS_CTS:
+	return "rtscts";
+
+    case GENSIO_SER_FLOWCONTROL_DCD:
+	return "dcd";
+
+    case GENSIO_SER_FLOWCONTROL_DTR:
+	return "dtr";
+
+    case GENSIO_SER_FLOWCONTROL_DSR:
+	return "dsr";
+
+    default:
+	return "?";
+    }
+}
+
+int
+gensio_str_to_flowcontrol(const char *sval)
+{
+    if (strcmp(sval, "none") == 0)
+	return GENSIO_SER_FLOWCONTROL_NONE;
+    if (strcmp(sval, "xonxoff") == 0)
+	return GENSIO_SER_FLOWCONTROL_XON_XOFF;
+    if (strcmp(sval, "rtscts") == 0)
+	return GENSIO_SER_FLOWCONTROL_RTS_CTS;
+    if (strcmp(sval, "dcd") == 0)
+	return GENSIO_SER_FLOWCONTROL_DCD;
+    if (strcmp(sval, "dtr") == 0)
+	return GENSIO_SER_FLOWCONTROL_DTR;
+    if (strcmp(sval, "dsr") == 0)
+	return GENSIO_SER_FLOWCONTROL_DSR;
+    return -1;
+}
+
+const char *
+gensio_onoff_to_str(unsigned int ival)
+{
+    switch(ival) {
+    case 0:
+	return "0";
+
+    case GENSIO_SER_ON:
+	return "on";
+
+    case GENSIO_SER_OFF:
+	return "off";
+
+    default:
+	return "?";
+    }
+}
+
+int
+gensio_str_to_onoff(const char *sval)
+{
+    if (strcmp(sval, "on") == 0)
+	return GENSIO_SER_ON;
+    if (strcmp(sval, "off") == 0)
+	return GENSIO_SER_OFF;
+    return -1;
+}
+
 /* For lack of a better place to put this. */
 bool gensio_uucp_locking_enabled = true;
