@@ -142,6 +142,7 @@ echon_deferred_op(struct gensio_runner *runner, void *cb_data)
     int err = 0;
 
     echon_lock(ndata);
+ restart:
     if (ndata->state == ECHON_IN_OPEN || ndata->state == ECHON_IN_OPEN_CLOSE) {
 	if (ndata->state == ECHON_IN_OPEN_CLOSE) {
 	    ndata->state = ECHON_IN_CLOSE;
@@ -202,6 +203,9 @@ echon_deferred_op(struct gensio_runner *runner, void *cb_data)
 	    ndata->close_done(ndata->io, ndata->close_data);
 	    echon_lock(ndata);
 	}
+
+	if (ndata->state != ECHON_CLOSED)
+	    goto restart;
     }
 
     ndata->deferred_op_pending = false;
@@ -412,14 +416,16 @@ echo_gensio_alloc(const void *gdata,
     gensiods max_read_size = GENSIO_DEFAULT_BUF_SIZE;
     bool noecho = false;
     const char *data = NULL;
+    GENSIO_DECLARE_PPGENSIO(p, o, cb, "echo", user_data);
 
     for (i = 0; args && args[i]; i++) {
-	if (gensio_check_keyds(args[i], "readbuf", &max_read_size) > 0)
+	if (gensio_pparm_ds(&p, args[i], "readbuf", &max_read_size) > 0)
 	    continue;
-	if (gensio_check_keybool(args[i], "noecho", &noecho) > 0)
+	if (gensio_pparm_bool(&p, args[i], "noecho", &noecho) > 0)
 	    continue;
-	if (gensio_check_keyvalue(args[i], "data", &data) > 0)
+	if (gensio_pparm_value(&p, args[i], "data", &data) > 0)
 	    continue;
+	gensio_pparm_unknown_parm(&p, args[i]);
 	return GE_INVAL;
     }
 
