@@ -211,7 +211,7 @@ static int
 telnet_ul_write(struct gensio_filter *filter,
 		gensio_ul_filter_data_handler handler, void *cb_data,
 		gensiods *rcount,
-		const struct gensio_sg *sg, gensiods sglen,
+		const struct gensio_sg *isg, gensiods sglen,
 		const char *const *auxdata)
 {
     struct telnet_filter *tfilter = filter_to_telnet(filter);
@@ -225,15 +225,15 @@ telnet_ul_write(struct gensio_filter *filter,
 	gensiods i, writelen = 0;
 
 	for (i = 0; i < sglen; i++) {
-	    size_t inlen = sg[i].buflen;
-	    const unsigned char *buf = sg[i].buf;
+	    size_t inlen = isg[i].buflen;
+	    const unsigned char *buf = isg[i].buf;
 
 	    tfilter->write_data_len =
 		process_telnet_xmit(tfilter->write_data,
 				    tfilter->max_write_size,
 				    &buf, &inlen);
-	    writelen += sg[i].buflen - inlen;
-	    if (inlen != sg[i].buflen)
+	    writelen += isg[i].buflen - inlen;
+	    if (inlen != isg[i].buflen)
 		break;
 	}
 	if (rcount)
@@ -583,10 +583,25 @@ telnet_filter_control(struct gensio_filter *filter, bool get, int op,
 	return 0;
 
     default:
+	if (tfilter->telnet_cbs->control)
+	    return tfilter->telnet_cbs->control(tfilter->handler_data,
+						get, op, data, datalen);
 	return GE_NOTSUP;
     }
 }
 
+static int
+telnet_filter_acontrol(struct gensio_filter *filter, bool get, int op,
+		       struct gensio_func_acontrol *data)
+{
+    struct telnet_filter *tfilter = filter_to_telnet(filter);
+
+    if (tfilter->telnet_cbs->acontrol)
+	return tfilter->telnet_cbs->acontrol(tfilter->handler_data,
+					     get, op, data);
+    return GE_NOTSUP;
+}
+
 static int gensio_telnet_filter_func(struct gensio_filter *filter, int op,
 				     void *func, void *data,
 				     gensiods *count,
@@ -642,6 +657,9 @@ static int gensio_telnet_filter_func(struct gensio_filter *filter, int op,
 	return telnet_filter_control(filter, *((bool *) cbuf), buflen, data,
 				     count);
 
+    case GENSIO_FILTER_FUNC_ACONTROL:
+	return telnet_filter_acontrol(filter, *((bool *) cbuf), buflen, data);
+
     default:
 	return GE_NOTSUP;
     }
