@@ -363,8 +363,10 @@ pty_check_close(void *handler_data, struct gensio_iod *iod,
     err = pty_check_exit_code(tdata);
     if (err == GE_INPROGRESS) {
 	/* FIXME - this should probably be configurable. */
-	if (tdata->check_close_count >= 500) /* Wait for 5 seconds. */
+	if (tdata->check_close_count >= 500) { /* Wait for 5 seconds. */
+	    err = GE_TIMEDOUT;
 	    goto out_finish;
+	}
 	tdata->check_close_count++;
 	timeout->secs = 0;
 	timeout->nsecs = 10000000;
@@ -782,7 +784,7 @@ pty_gensio_alloc(const void *gdata, const char * const args[],
     }
 
     tdata->ll = fd_gensio_ll_alloc(o, NULL, &pty_fd_ll_ops, tdata,
-				   max_read_size, false);
+				   max_read_size, false, false);
     if (!tdata->ll)
 	goto out_nomem;
 
