@@ -304,9 +304,9 @@ gensio_os_sctp_open_sockets(struct gensio_os_funcs *o,
 
  out_err:
     sctp_shutdown_fds(o, fds, nr_fds);
+#if !HAVE_WORKING_PORT0
     fds = NULL;
     nr_fds = 0;
-#if !HAVE_WORKING_PORT0
     if (rv == GE_ADDRINUSE && scaninfo.start != 0 &&
 		scaninfo.curr != scaninfo.start) {
 	/* We need to keep scanning. */
@@ -1020,10 +1020,10 @@ gensio_stdsock_recvfrom(struct gensio_iod *iod,
 		}
 	    }
 	}
-	gensio_addr_rewind(addr);
 #endif
     }
 #endif
+    gensio_addr_rewind(addr);
     if (!err && rcount)
 	*rcount = rv;
     return gensio_os_err_to_err(o, err);
@@ -1231,8 +1231,6 @@ gensio_stdsock_socket_set_setup(struct gensio_iod *iod,
     }
 
     if (opensock_flags & GENSIO_SET_OPENSOCK_NODELAY) {
-	struct gensio_stdsock_info *gsi = NULL;
-
 	err = o->iod_control(iod, GENSIO_IOD_CONTROL_SOCKINFO, true,
 			     (intptr_t) &gsi);
 	if (err)
@@ -1430,7 +1428,7 @@ gensio_stdsock_close_socket(struct gensio_iod *iod, bool retry, bool force)
 #else
     err = close_socket(o, o->iod_get_fd(iod));
 #endif
-    if (err != GE_INPROGRESS && gsi)
+    if (err != GE_INPROGRESS)
 	o->free(o, gsi);
     return err;
 }
@@ -2260,10 +2258,22 @@ gensio_setup_listen_socket(struct gensio_os_funcs *o, bool do_listen,
 	    if (bind(fd, addr, addrlen) == 0) {
 		goto got_it;
 	    } else {
-		if (sock_errno != SOCK_EADDRINUSE)
-		    goto out_err;
+#ifdef _WIN32
+		/*
+		 * Apparently, if you try to grab some ports on
+		 * windows, you get a permission error if maybe some
+		 * other process that is privileged or just another
+		 * user is using it?  Sometimes this happens, just let
+		 * it go.
+		 */
+		if (sock_errno == WSAEACCES)
+		    goto next_port;
+#endif
+		if (sock_errno == SOCK_EADDRINUSE)
+		    goto next_port;
+		goto out_err;
 	    }
-
+	next_port:
 	    si->curr = gensio_dyn_scan_next(si->curr);
 	} while (si->curr != si->start);
 	/* Unable to find an open port, give up. */
