@@ -36,6 +36,7 @@
 
 struct gensio_data {
     struct selector_s *sel;
+    unsigned int flags;
     lock_type reflock;
     unsigned int refcount;
     bool freesel;
@@ -48,16 +49,17 @@ static void *
 gensio_unix_zalloc(struct gensio_os_funcs *o, gensiods size)
 {
     struct gensio_data *d = o->user_data;
-
-    return gensio_i_zalloc(d->mtrack, size);
+    TRACE_MEM;
+    return gensio_i_zalloc(d->mtrack, size,
+			   TRACE_MEM_CALLERS, TRACE_MEM_CALLERS_SIZE);
 }
 
 static void
 gensio_unix_free(struct gensio_os_funcs *o, void *v)
 {
     struct gensio_data *d = o->user_data;
-
-    gensio_i_free(d->mtrack, v);
+    TRACE_MEM;
+    gensio_i_free(d->mtrack, v, TRACE_MEM_CALLERS, TRACE_MEM_CALLERS_SIZE);
 }
 
 static void
@@ -98,6 +100,25 @@ timeval_to_gensio_time(gensio_time *t, struct timeval *tv)
 
 #include <pthread.h>
 
+static int
+init_mutex(unsigned int flags, pthread_mutex_t *mutex)
+{
+#ifdef USE_PTHREADS
+    pthread_mutexattr_t mattr;
+
+    pthread_mutexattr_init(&mattr);
+    if (flags & GENSIO_OS_FUNCS_FLAG_PRIO_INHERIT) {
+	int rv = pthread_mutexattr_setprotocol(&mattr, PTHREAD_PRIO_INHERIT);
+	if (rv)
+	    return rv;
+    }
+    pthread_mutex_init(mutex, &mattr);
+#else
+    LOCK_INIT(mutex);
+#endif
+    return 0;
+}
+
 struct waiter_data {
     pthread_t tid;
     int wake_sig;
@@ -119,13 +140,14 @@ static waiter_t *
 alloc_waiter(struct gensio_os_funcs *o, struct selector_s *sel, int wake_sig)
 {
     waiter_t *waiter;
+    struct gensio_data *d = o->user_data;
 
     waiter = o->zalloc(o, sizeof(waiter_t));
     if (waiter) {
 	waiter->o = o;
 	waiter->wake_sig = wake_sig;
 	waiter->sel = sel;
-	pthread_mutex_init(&waiter->lock, NULL);
+	init_mutex(d->flags, &waiter->lock);
 	waiter->wts.next = &waiter->wts;
 	waiter->wts.prev = &waiter->wts;
     }
@@ -336,10 +358,14 @@ static struct gensio_lock *
 gensio_unix_alloc_lock(struct gensio_os_funcs *f)
 {
     struct gensio_lock *lock = f->zalloc(f, sizeof(*lock));
+    struct gensio_data *d = f->user_data;
 
     if (lock) {
 	lock->f = f;
-	LOCK_INIT(&lock->lock);
+	if (init_mutex(d->flags, &lock->lock)) {
+	    f->free(f, lock);
+	    return NULL;
+	}
     }
 
     return lock;
@@ -388,6 +414,7 @@ struct gensio_iod_unix {
     struct gensio_iod r;
     int orig_fd;
     int fd;
+    int sfd;
     enum gensio_iod_type type;
     bool handlers_set;
     bool is_stdio;
@@ -655,7 +682,10 @@ gensio_unix_alloc_timer(struct gensio_os_funcs *f,
     timer->f = f;
     timer->handler = handler;
     timer->cb_data = cb_data;
-    LOCK_INIT(&timer->lock);
+    if (init_mutex(d->flags, &timer->lock)) {
+	f->free(f, timer);
+	return NULL;
+    }
 
     rv = sel_alloc_timer(d->sel, gensio_timeout_handler, timer,
 			 &timer->sel_timer);
@@ -948,6 +978,7 @@ gensio_unix_get_wake_sig(struct gensio_os_funcs *f)
 static lock_type defos_lock = LOCK_INITIALIZER;
 static struct gensio_os_funcs *defoshnd;
 static int defoshnd_wake_sig = -1;
+static bool proc_setup;
 
 static struct gensio_os_funcs *
 gensio_unix_get_funcs(struct gensio_os_funcs *f)
@@ -989,22 +1020,6 @@ gensio_unix_free_funcs(struct gensio_os_funcs *f)
     free(f);
 }
 
-static lock_type once_lock = LOCK_INITIALIZER;
-
-static void
-gensio_unix_call_once(struct gensio_os_funcs *f, struct gensio_once *once,
-		      void (*func)(void *cb_data), void *cb_data)
-{
-    if (once->called)
-	return;
-    LOCK(&once_lock);
-    if (!once->called) {
-	once->called = true;
-	func(cb_data);
-    }
-    UNLOCK(&once_lock);
-}
-
 static void
 gensio_unix_get_monotonic_time(struct gensio_os_funcs *f, gensio_time *time)
 {
@@ -1024,11 +1039,11 @@ gensio_handle_fork(struct gensio_os_funcs *f)
 
 static int
 gensio_unix_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
-		    intptr_t ofd, struct gensio_iod **riod)
+		    intptr_t ofd, struct gensio_iod **riod, ...)
 {
     struct gensio_iod_unix *iod = NULL;
     bool closefd = false;
-    int err = GE_NOMEM, fd = ofd;
+    int err = GE_NOMEM, fd = ofd, sfd = -1;
 
     if (type >= NR_GENSIO_IOD_TYPES)
 	return GE_INVAL;
@@ -1044,7 +1059,7 @@ gensio_unix_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 	    return gensio_os_err_to_err(o, errno);
 	closefd = true;
     } else if (type == GENSIO_IOD_PTY) {
-	err = gensio_unix_pty_alloc(o, &fd);
+      err = gensio_unix_pty_alloc(o, &fd, &sfd);
 	if (err)
 	    return err;
 	closefd = true;
@@ -1057,6 +1072,7 @@ gensio_unix_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
     }
     iod->r.f = o;
     iod->fd = fd;
+    iod->sfd = sfd;
     iod->orig_fd = ofd;
     if (type == GENSIO_IOD_STDIO) {
 	struct stat statb;
@@ -1099,8 +1115,11 @@ gensio_unix_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
  out_err:
     if (iod)
 	o->free(o, iod);
-    else if (closefd)
+    else if (closefd) {
 	close(fd);
+	if (sfd != -1)
+	    close(sfd);
+    }
     return err;
 }
 
@@ -1235,6 +1254,10 @@ gensio_unix_close(struct gensio_iod **iodp)
 	assert(err == 0);
 #endif
     }
+    if (iod->sfd != -1) {
+	close(iod->sfd);
+	iod->sfd = -1;
+    }
     o->release_iod(iiod);
     *iodp = NULL;
 
@@ -1428,7 +1451,7 @@ gensio_unix_pty_control(struct gensio_iod_unix *iod, int op, bool get,
 	return 0;
 
     case GENSIO_IOD_CONTROL_START:
-	return gensio_unix_pty_start(o, iod->fd, iod->u.pty.argv,
+	return gensio_unix_pty_start(o, iod->fd, &iod->sfd, iod->u.pty.argv,
 				     iod->u.pty.env, iod->u.pty.start_dir,
 				     &iod->u.pty.pid);
 
@@ -1575,7 +1598,7 @@ gensio_unix_control(struct gensio_os_funcs *o, int func, void *data,
 }
 
 static struct gensio_os_funcs *
-gensio_unix_alloc_sel(struct selector_s *sel, int wake_sig)
+gensio_unix_alloc_sel(struct selector_s *sel, int wake_sig, unsigned int flags)
 {
     struct gensio_data *d;
     struct gensio_os_funcs *o;
@@ -1591,7 +1614,12 @@ gensio_unix_alloc_sel(struct selector_s *sel, int wake_sig)
 	return NULL;
     }
     memset(d, 0, sizeof(*d));
-    LOCK_INIT(&d->reflock);
+    d->flags = flags;
+    if (init_mutex(d->flags, &d->reflock)) {
+	free(d);
+	free(o);
+	return NULL;
+    }
     d->refcount = 1;
 
     o->user_data = d;
@@ -1630,7 +1658,7 @@ gensio_unix_alloc_sel(struct selector_s *sel, int wake_sig)
     o->get_wake_sig = gensio_unix_get_wake_sig;
     o->get_funcs = gensio_unix_get_funcs;
     o->free_funcs = gensio_unix_free_funcs;
-    o->call_once = gensio_unix_call_once;
+    o->call_once = gensio_call_once;
     o->get_monotonic_time = gensio_unix_get_monotonic_time;
     o->handle_fork = gensio_handle_fork;
     o->add_iod = gensio_unix_add_iod;
@@ -1741,6 +1769,50 @@ winch_sig_handler(int sig)
 }
 #endif
 
+int
+gensio_os_thread_setup(struct gensio_os_funcs *o)
+{
+    int rv = 0;
+#ifdef USE_PTHREADS
+    sigset_t sigs, old_sigs;
+    int wake_sig = o->get_wake_sig(o);
+    struct sigaction sigdo;
+
+    sigemptyset(&sigs);
+    if (wake_sig)
+	sigaddset(&sigs, wake_sig);
+    sigaddset(&sigs, SIGCHLD); /* Ignore SIGCHLD in normal operation. */
+    sigaddset(&sigs, SIGPIPE); /* Ignore broken pipes. */
+#if HAVE_DECL_SIGWINCH
+    /*
+     * SIGWINCH is set here, not in the register_winsize_handler call, because
+     * we can't modify the signal set after initial startup.
+     */
+    sigaddset(&sigs, SIGWINCH);
+#endif
+    rv = pthread_sigmask(SIG_BLOCK, &sigs, &old_sigs);
+    if (rv) {
+	rv = gensio_os_err_to_err(o, errno);
+	goto out;
+    }
+
+    if (wake_sig) {
+	memset(&sigdo, 7, sizeof(sigdo));
+	sigdo.sa_handler = handle_wakesig;
+	sigdo.sa_flags = 0;
+	rv = sigaction(wake_sig, &sigdo, NULL);
+	if (rv) {
+	    rv = gensio_os_err_to_err(o, errno);
+	    pthread_sigmask(SIG_SETMASK, &old_sigs, NULL);
+	    goto out;
+	}
+    }
+
+ out:
+#endif
+    return rv;
+}
+
 int
 gensio_os_proc_setup(struct gensio_os_funcs *o,
 		     struct gensio_os_proc_data **rdata)
@@ -1748,7 +1820,11 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
     struct gensio_os_proc_data *data;
     sigset_t sigs;
     struct sigaction sigdo;
-    int rv;
+    int rv = GE_INUSE;
+
+    LOCK(&defos_lock);
+    if (proc_setup)
+	goto out;
 
     data = &proc_data;
     data->o = o;
@@ -1761,10 +1837,17 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
 	sigaddset(&sigs, data->wake_sig);
     sigaddset(&sigs, SIGCHLD); /* Ignore SIGCHLD in normal operation. */
     sigaddset(&sigs, SIGPIPE); /* Ignore broken pipes. */
+#if HAVE_DECL_SIGWINCH
+    /*
+     * SIGWINCH is set here, not in the register_winsize_handler call, because
+     * we can't modify the signal set after initial startup.
+     */
+    sigaddset(&sigs, SIGWINCH);
+#endif
     rv = sigprocmask(SIG_BLOCK, &sigs, &data->old_sigs);
     if (rv) {
 	rv = gensio_os_err_to_err(o, errno);
-	return rv;
+	goto out;
     }
     data->wait_sigs = data->old_sigs;
     if (data->wake_sig)
@@ -1772,6 +1855,9 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
     sigdelset(&data->wait_sigs, SIGCHLD); /* Allow SIGCHLD while waiting. */
     sigaddset(&data->check_sigs, SIGCHLD);
     sigaddset(&data->wait_sigs, SIGPIPE); /* No SIGPIPE ever. */
+#if HAVE_DECL_SIGWINCH
+    sigdelset(&data->wait_sigs, SIGWINCH); /* See note above on SIGWINCH */
+#endif
 
     memset(&sigdo, 0, sizeof(sigdo));
     sigdo.sa_handler = handle_sigchld;
@@ -1780,7 +1866,7 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
     if (rv) {
 	rv = gensio_os_err_to_err(o, errno);
 	sigprocmask(SIG_SETMASK, &data->old_sigs, NULL);
-	return rv;
+	goto out;
     }
 
     if (data->wake_sig) {
@@ -1791,7 +1877,7 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
 	    rv = gensio_os_err_to_err(o, errno);
 	    sigaction(SIGCHLD, &data->old_sigchld, NULL);
 	    sigprocmask(SIG_SETMASK, &data->old_sigs, NULL);
-	    return rv;
+	    goto out;
 	}
     }
 
@@ -1801,13 +1887,16 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
 	sigprocmask(SIG_SETMASK, &data->old_sigs, NULL);
 	if (data->wake_sig)
 	    sigaction(data->wake_sig, &data->old_wakesig, NULL);
-	return rv;
+	goto out;
     }
 
     LOCK_INIT(&data->handler_lock);
 
     *rdata = data;
-    return 0;
+    proc_setup = true;
+ out:
+    UNLOCK(&defos_lock);
+    return rv;
 }
 
 void
@@ -1846,7 +1935,7 @@ check_for_sigpending(sigset_t *check_for)
     for (sig = 0; sig < NSIG; sig++) {
 	if (sigismember(&pending, sig) && sigismember(check_for, sig)) {
 	    rsig = sig;
-	    sigwait(&check_for, &sig);
+	    sigwait(check_for, &sig);
 	    break;
 	}
     }
@@ -1857,7 +1946,11 @@ check_for_sigpending(sigset_t *check_for)
 void
 gensio_os_proc_cleanup(struct gensio_os_proc_data *data)
 {
-    int sig;
+    LOCK(&defos_lock);
+    if (!proc_setup)
+	goto out;
+
+    proc_setup = false;
 
     /* We should be single-threaded here. */
     while (data->cleanup_handlers) {
@@ -1890,10 +1983,13 @@ gensio_os_proc_cleanup(struct gensio_os_proc_data *data)
     sigaction(SIGCHLD, &data->old_sigchld, NULL);
 
     /* Clear out any pending signals before we restore the mask. */
-    while ((sig = check_for_sigpending(&data->check_sigs)) > 0)
+    while (check_for_sigpending(&data->check_sigs) > 0)
 	;
 
     sigprocmask(SIG_SETMASK, &data->old_sigs, NULL);
+ out:
+    UNLOCK(&defos_lock);
+    return;
 }
 
 void
@@ -2070,27 +2166,25 @@ gensio_os_proc_register_winsize_handler(struct gensio_os_proc_data *data,
 {
 #if HAVE_DECL_SIGWINCH
     struct gensio_iod_unix *iod = i_to_sel(console_iod);
-    int err;
+    int err = 0;
     struct sigaction act;
-    sigset_t sigs, old_sigs;
     struct winsize win;
 
+    LOCK(&data->handler_lock);
     if (data->winch_sig_set) {
 	data->winch_sig_set = false;
 	sigaction(SIGWINCH, &data->old_sigwinch, NULL);
     }
-    if (!handler)
-	return 0;
+    if (!handler) {
+	data->got_winch_sig = false;
+	goto out_unlock;
+    }
 
     err = ioctl(iod->fd, TIOCGWINSZ, &win);
-    if (err == -1)
-	return GE_NOTSUP;
-
-    sigemptyset(&sigs);
-    sigaddset(&sigs, SIGWINCH);
-    err = sigprocmask(SIG_BLOCK, &sigs, &old_sigs);
-    if (err)
-	return gensio_os_err_to_err(data->o, errno);
+    if (err == -1) {
+	err = GE_NOTSUP;
+	goto out_unlock;
+    }
 
     data->winch_handler = handler;
     data->winch_handler_data = handler_data;
@@ -2100,17 +2194,24 @@ gensio_os_proc_register_winsize_handler(struct gensio_os_proc_data *data,
     act.sa_handler = winch_sig_handler;
     err = sigaction(SIGWINCH, &act, &data->old_sigwinch);
     if (err) {
-	err = errno;
-	goto out_err;
+	err = gensio_os_err_to_err(data->o, errno);
+	goto out_unlock;
     }
     sigdelset(&data->wait_sigs, SIGWINCH);
     data->winch_sig_set = true;
+    /*
+     * For at least MacOS, using kill() to self doesn't seem to work
+     * properly.  pthread_kill() seems to work ok.
+     */
+#ifdef USE_PTHREADS
+    pthread_kill(pthread_self(), SIGWINCH);
+#else
     kill(getpid(), SIGWINCH);
-    return 0;
+#endif
 
- out_err:
-    sigprocmask(SIG_SETMASK, &old_sigs, NULL);
-    return gensio_os_err_to_err(data->o, err);
+ out_unlock:
+    UNLOCK(&data->handler_lock);
+    return err;
 #else
     return GE_NOTSUP;
 #endif
@@ -2237,11 +2338,12 @@ static sel_lock_t *
 defsel_lock_alloc(void *cb_data)
 {
     sel_lock_t *l;
+    unsigned int flags = (intptr_t) cb_data;
 
     l = malloc(sizeof(*l));
     if (!l)
 	return NULL;
-    LOCK_INIT(&l->lock);
+    init_mutex(flags, &l->lock);
     return l;
 }
 
@@ -2266,20 +2368,30 @@ defsel_unlock(sel_lock_t *l)
 
 #endif
 
-int
-gensio_unix_funcs_alloc(struct selector_s *sel, int wake_sig,
-			struct gensio_os_funcs **ro)
+static int
+i_gensio_unix_funcs_alloc(struct selector_s *sel, int wake_sig,
+			  unsigned int flags,
+			  struct gensio_os_funcs **ro)
 {
     struct gensio_os_funcs *o;
     bool freesel = false;
     int rv;
 
+    if (flags & ~GENSIO_OS_FUNCS_FLAG_PRIO_INHERIT)
+	return GE_NOTSUP;
+
+#ifndef USE_PTHREADS
+    if (flags & GENSIO_OS_FUNCS_FLAG_PRIO_INHERIT)
+	return GE_NOTSUP;
+#endif
+
     if (!sel) {
 #ifdef USE_PTHREADS
 	rv = sel_alloc_selector_thread(&sel, wake_sig,
 				       defsel_lock_alloc,
 				       defsel_lock_free, defsel_lock,
-				       defsel_unlock, NULL);
+				       defsel_unlock,
+				       (void *) (intptr_t) flags);
 #else
 	rv = sel_alloc_selector_nothread(&sel);
 #endif
@@ -2288,7 +2400,7 @@ gensio_unix_funcs_alloc(struct selector_s *sel, int wake_sig,
 	freesel = true;
     }
 
-    o = gensio_unix_alloc_sel(sel, wake_sig);
+    o = gensio_unix_alloc_sel(sel, wake_sig, flags);
     if (o) {
 	struct gensio_data *d = o->user_data;
 
@@ -2301,6 +2413,13 @@ gensio_unix_funcs_alloc(struct selector_s *sel, int wake_sig,
     return 0;
 }
 
+int
+gensio_unix_funcs_alloc(struct selector_s *sel, int wake_sig,
+			struct gensio_os_funcs **ro)
+{
+    return i_gensio_unix_funcs_alloc(sel, wake_sig, 0, ro);
+}
+
 struct gensio_os_funcs *
 gensio_selector_alloc(struct selector_s *sel, int wake_sig)
 {
@@ -2321,7 +2440,7 @@ gensio_default_os_hnd(int wake_sig, struct gensio_os_funcs **o)
 {
     int err = 0;
 
-    if (wake_sig == -198234)
+    if (wake_sig == GENSIO_OS_FUNCS_DEFAULT_THREAD_SIGNAL)
 	wake_sig = SIGUSR1;
 
     LOCK(&defos_lock);
@@ -2344,6 +2463,32 @@ gensio_default_os_hnd(int wake_sig, struct gensio_os_funcs **o)
     return err;
 }
 
+int
+gensio_valloc_os_funcs(int wake_sig, struct gensio_os_funcs **o,
+		       unsigned int flags, va_list ap)
+{
+    if (wake_sig == GENSIO_OS_FUNCS_DEFAULT_THREAD_SIGNAL)
+	wake_sig = SIGUSR1;
+
+    if (flags)
+	return GE_NOTSUP;
+
+    return gensio_unix_funcs_alloc(NULL, wake_sig, o);
+}
+
+int
+gensio_alloc_os_funcs(int wake_sig, struct gensio_os_funcs **o,
+		      unsigned int flags, ...)
+{
+    va_list ap;
+    int rv;
+
+    va_start(ap, flags);
+    rv = gensio_valloc_os_funcs(wake_sig, o, flags, ap);
+    va_end(ap);
+    return rv;
+}
+
 void
 gensio_osfunc_exit(int rv)
 {
