@@ -181,8 +181,8 @@ typedef struct sel_wait_list_s
 
     /* The time when the thread is set to wake up. */
     struct timeval wake_time;
+    volatile struct timespec wait_time;
 #ifdef BROKEN_PSELECT
-    struct timespec *wait_time;
     bool signalled;
 #endif
     struct sel_wait_list_s *next, *prev;
@@ -287,8 +287,8 @@ i_wake_sel_thread(struct selector_s *sel, struct timeval *new_timeout)
 	{
 #ifdef BROKEN_PSELECT
 	    item->signalled = true;
-	    item->wait_time->tv_sec = 0;
-	    item->wait_time->tv_nsec = 0;
+	    item->wait_time.tv_sec = 0;
+	    item->wait_time.tv_nsec = 0;
 #endif
 	    item->send_sig(item->thread_id, item->send_sig_cb_data);
 	}
@@ -313,8 +313,8 @@ i_sel_wake_first(struct selector_s *sel)
     if (item->send_sig && item != &sel->wait_list) {
 #ifdef BROKEN_PSELECT
 	item->signalled = true;
-	item->wait_time->tv_sec = 0;
-	item->wait_time->tv_nsec = 0;
+	item->wait_time.tv_sec = 0;
+	item->wait_time.tv_nsec = 0;
 #endif
 	item->send_sig(item->thread_id, item->send_sig_cb_data);
     }
@@ -335,8 +335,8 @@ sel_wake_one(struct selector_s *sel, long thread_id, sel_send_sig_cb killer,
     while (item != &sel->wait_list) {
 	if (thread_id == item->thread_id) {
 	    item->signalled = true;
-	    item->wait_time->tv_sec = 0;
-	    item->wait_time->tv_nsec = 0;
+	    item->wait_time.tv_sec = 0;
+	    item->wait_time.tv_nsec = 0;
 	    break;
 	}
 	item = item->next;
@@ -363,14 +363,13 @@ add_sel_wait_list(struct selector_s *sel, sel_wait_list_t *item,
 		  sel_send_sig_cb send_sig,
 		  void            *cb_data,
 		  long thread_id,
-		  struct timeval *wake_time, struct timespec *wait_time)
+		  struct timeval *wake_time)
 {
     item->thread_id = thread_id;
     item->send_sig = send_sig;
     item->send_sig_cb_data = cb_data;
     item->wake_time = *wake_time;
 #ifdef BROKEN_PSELECT
-    item->wait_time = wait_time;
     item->signalled = false;
 #endif
     item->next = sel->wait_list.next;
@@ -589,9 +588,9 @@ i_sel_clear_fd_handler(struct selector_s *sel, int fd, int rpt)
     if (sel->epollfd < 0)
 #endif
     {
-	FD_CLR(fd, &sel->read_set);
-	FD_CLR(fd, &sel->write_set);
-	FD_CLR(fd, &sel->except_set);
+	FD_CLR(fd, (fd_set *) &sel->read_set);
+	FD_CLR(fd, (fd_set *) &sel->write_set);
+	FD_CLR(fd, (fd_set *) &sel->except_set);
     }
 
     /* Move maxfd down if necessary. */
@@ -651,7 +650,7 @@ sel_set_fd_read_handler(struct selector_s *sel, int fd, int state)
 #ifdef HAVE_EPOLL_PWAIT
 	if (sel->epollfd < 0)
 #endif
-	    FD_SET(fd, &sel->read_set);
+	    FD_SET(fd, (fd_set *) &sel->read_set);
     } else if (state == SEL_FD_HANDLER_DISABLED) {
 	if (!fdc->read_enabled)
 	    goto out;
@@ -659,7 +658,7 @@ sel_set_fd_read_handler(struct selector_s *sel, int fd, int state)
 #ifdef HAVE_EPOLL_PWAIT
 	if (sel->epollfd < 0)
 #endif
-	    FD_CLR(fd, &sel->read_set);
+	    FD_CLR(fd, (fd_set *) &sel->read_set);
     }
     if (sel_update_fd(sel, fdc, EPOLL_CTL_MOD))
 	sel_wake_all(sel);
@@ -688,7 +687,7 @@ sel_set_fd_write_handler(struct selector_s *sel, int fd, int state)
 #ifdef HAVE_EPOLL_PWAIT
 	if (sel->epollfd < 0)
 #endif
-	    FD_SET(fd, &sel->write_set);
+	    FD_SET(fd, (fd_set *) &sel->write_set);
     } else if (state == SEL_FD_HANDLER_DISABLED) {
 	if (!fdc->write_enabled)
 	    goto out;
@@ -696,7 +695,7 @@ sel_set_fd_write_handler(struct selector_s *sel, int fd, int state)
 #ifdef HAVE_EPOLL_PWAIT
 	if (sel->epollfd < 0)
 #endif
-	    FD_CLR(fd, &sel->write_set);
+	    FD_CLR(fd, (fd_set *) &sel->write_set);
     }
     if (sel_update_fd(sel, fdc, EPOLL_CTL_MOD))
 	sel_wake_all(sel);
@@ -725,7 +724,7 @@ sel_set_fd_except_handler(struct selector_s *sel, int fd, int state)
 #ifdef HAVE_EPOLL_PWAIT
 	if (sel->epollfd < 0)
 #endif
-	    FD_SET(fd, &sel->except_set);
+	    FD_SET(fd, (fd_set *) &sel->except_set);
     } else if (state == SEL_FD_HANDLER_DISABLED) {
 	if (!fdc->except_enabled)
 	    goto out;
@@ -733,7 +732,7 @@ sel_set_fd_except_handler(struct selector_s *sel, int fd, int state)
 #ifdef HAVE_EPOLL_PWAIT
 	if (sel->epollfd < 0)
 #endif
-	    FD_CLR(fd, &sel->except_set);
+	    FD_CLR(fd, (fd_set *) &sel->except_set);
     }
     if (sel_update_fd(sel, fdc, EPOLL_CTL_MOD))
 	sel_wake_all(sel);
@@ -1108,7 +1107,7 @@ handle_selector_call(struct selector_s *sel, fd_control_t *fdc,
 	/* Somehow we don't have a handler for this.
 	   Just shut it down. */
 	if (fdset)
-	    FD_CLR(fdc->fd, fdset);
+	    FD_CLR(fdc->fd, (fd_set *) fdset);
 	return;
     }
 
@@ -1160,8 +1159,8 @@ setup_my_sigmask(sigset_t *sigmask, sigset_t *isigmask)
  * 	  <  0  when error
  */
 static int
-process_fds(struct selector_s	    *sel,
-	    volatile struct timespec *timeout,
+process_fds(struct selector_s *sel,
+	    sel_wait_list_t *item,
 	    sigset_t *isigmask)
 {
     fd_set      tmp_read_set;
@@ -1188,7 +1187,7 @@ process_fds(struct selector_s	    *sel,
 		  &tmp_read_set,
 		  &tmp_write_set,
 		  &tmp_except_set,
-		  (struct timespec *) timeout, &sigmask);
+		  (struct timespec *) &item->wait_time, &sigmask);
     if (err < 0) {
 	if (errno == EBADF || errno == EBADFD)
 	    /* We raced, just retry it. */
@@ -1221,7 +1220,7 @@ process_fds(struct selector_s	    *sel,
     }
  out_unlock:
     sel_fd_unlock(sel);
-out:
+ out:
     return err;
 }
 
@@ -1341,7 +1340,6 @@ sel_select_intr_sigmask(struct selector_s *sel,
 {
     int             err = 0, old_errno;
     struct timeval  wake_time, tmp_timeout;
-    struct timespec loc_timeout;
     sel_wait_list_t wait_entry;
     unsigned int    count;
     struct timeval  end = { 0, 0 }, now;
@@ -1365,30 +1363,34 @@ sel_select_intr_sigmask(struct selector_s *sel,
 	    }
 	}
 
-	loc_timeout.tv_sec = tmp_timeout.tv_sec;
-	loc_timeout.tv_nsec = tmp_timeout.tv_usec * 1000;
+	memset(&wait_entry, 0, sizeof(wait_entry));
+
+	wait_entry.wait_time.tv_sec = tmp_timeout.tv_sec;
+	wait_entry.wait_time.tv_nsec = tmp_timeout.tv_usec * 1000;
 
 	add_sel_wait_list(sel, &wait_entry, send_sig, cb_data, thread_id,
-			  &wake_time, &loc_timeout);
+			  &wake_time);
 	sel_timer_unlock(sel);
 
 #ifdef HAVE_EPOLL_PWAIT
 	if (sel->epollfd >= 0)
-	    err = process_fds_epoll(sel, &loc_timeout, sigmask);
+	    err = process_fds_epoll(sel,
+				    (struct timespec *) &wait_entry.wait_time,
+				    sigmask);
 	else
 #endif
-	    err = process_fds(sel, &loc_timeout, sigmask);
+	    err = process_fds(sel, &wait_entry, sigmask);
 
 	old_errno = errno;
 
 	sel_timer_lock(sel);
-	if (err == 0) {
 #ifdef BROKEN_PSELECT
-	    if (wait_entry.signalled) {
-		err = -1;
-		old_errno = EINTR;
-	    } else
+	if (wait_entry.signalled && !err) {
+	    err = -1;
+	    old_errno = EINTR;
+	}
 #endif
+	if (err == 0) {
 	    if (!user_timeout) {
 		/*
 		 * Only return a timeout if we waited on the user's timeout
@@ -1505,9 +1507,9 @@ sel_alloc_selector_thread(struct selector_s **new_selector, int wake_sig,
 
     sel->wake_sig = wake_sig;
 
-    FD_ZERO((fd_set *) &sel->read_set);
-    FD_ZERO((fd_set *) &sel->write_set);
-    FD_ZERO((fd_set *) &sel->except_set);
+    FD_ZERO((fd_set *) (fd_set *) &sel->read_set);
+    FD_ZERO((fd_set *) (fd_set *) &sel->write_set);
+    FD_ZERO((fd_set *) (fd_set *) &sel->except_set);
 
     memset(sel->fds, 0, sizeof(sel->fds));
 
@@ -1529,9 +1531,14 @@ sel_alloc_selector_thread(struct selector_s **new_selector, int wake_sig,
 
     sigemptyset(&sigset);
     sigaddset(&sigset, wake_sig);
+#ifdef USE_PTHREADS
+    rv = pthread_sigmask(SIG_BLOCK, &sigset, NULL);
+#else
     rv = sigprocmask(SIG_BLOCK, &sigset, NULL);
-    if (rv == -1) {
+    if (rv == -1)
 	rv = errno;
+#endif
+    if (rv) {
 	if (sel->sel_lock_alloc) {
 	    sel->sel_lock_free(sel->fd_lock);
 		sel->sel_lock_free(sel->timer_lock);
