@@ -16,9 +16,8 @@
 
 #include <gensio/sergensio_class.h>
 #include <gensio/gensio_ll_fd.h>
-#include <gensio/gensio_osops.h>
 
-#include "uucplock.h"
+#include "seriallock.h"
 #include "utils.h"
 
 static int
@@ -80,15 +79,23 @@ parity_to_str(int val)
 
 struct sterm_data;
 
+struct termio_xlat_str {
+    const char *sval;
+    int ival;
+};
+
 struct termio_op_q {
     int op;
     int (*xlat)(struct sterm_data *, bool get, int *oval, int val);
+    gensio_control_done cdone;
     void (*done)(struct sergensio *sio, int err, int val, void *cb_data);
     void *cb_data;
+    const struct termio_xlat_str *xlatstr;
     struct termio_op_q *next;
 };
 
 struct sterm_data {
+    struct gensio *io;
     struct sergensio *sio;
     struct gensio_os_funcs *o;
 
@@ -121,9 +128,12 @@ struct sterm_data {
      */
     bool is_pty;
 
-    bool write_only;		/* No serial settings, no read. */
+    bool write_only;		/* No read. */
+    bool read_only;		/* No write. */
+    bool set_tty;		/* No serial settings. */
 
-    bool no_uucp_lock;
+    bool uucp_lock;
+    bool flock_lock;
 
     void *default_sercfg;
     int def_baud;
@@ -246,6 +256,31 @@ sterm_start_deferred_op(struct sterm_data *sdata)
     }
 }
 
+static void
+serconf_call_cdone(struct sterm_data *sdata, struct termio_op_q *qe,
+		   int err, int val)
+{
+    unsigned int i;
+    const char *sval = NULL;
+    char str[20];
+
+    if (!err) {
+	if (qe->xlatstr) {
+	    for (i = 0; qe->xlatstr[i].sval; i++) {
+		if (val == qe->xlatstr[i].ival) {
+		    sval = qe->xlatstr[i].sval;
+		    break;
+		}
+	    }
+	}
+	if (!sval) {
+	    snprintf(str, sizeof(str), "%d", val);
+	    sval = str;
+	}
+    }
+    qe->cdone(sdata->io, err, sval, sval ? strlen(sval): 0, qe->cb_data);
+}
+
 static void
 serconf_process(struct sterm_data *sdata)
 {
@@ -259,7 +294,10 @@ serconf_process(struct sterm_data *sdata)
 	if (!err && qe->xlat)
 	    err = qe->xlat(sdata, true, &val, val);
 	sterm_unlock(sdata);
-	qe->done(sdata->sio, err, val, qe->cb_data);
+	if (qe->cdone)
+	    serconf_call_cdone(sdata, qe, err, val);
+	else
+	    qe->done(sdata->sio, err, val, qe->cb_data);
 	sdata->o->free(sdata->o, qe);
 	sterm_lock(sdata);
     }
@@ -277,9 +315,11 @@ serconf_clear_q(struct sterm_data *sdata)
 }
 
 static int
-serconf_set_get(struct sterm_data *sdata, int op, int val,
+serconf_set_get(struct sterm_data *sdata, int op, int val, const char *sval,
 		int (*xlat)(struct sterm_data *sdata, bool get,
 			    int *oval, int val),
+		gensio_control_done cdone,
+		const struct termio_xlat_str *xlatstr,
 		void (*done)(struct sergensio *sio, int err,
 			     int val, void *cb_data),
 		void *cb_data)
@@ -287,14 +327,32 @@ serconf_set_get(struct sterm_data *sdata, int op, int val,
     struct termio_op_q *qe = NULL;
     int err = 0;
 
-    if (sdata->write_only)
+    if (!sdata->set_tty)
 	return GE_NOTSUP;
 
-    if (done) {
+    if (sval) {
+	if (xlatstr) {
+	    unsigned int i;
+
+	    for (i = 0; xlatstr[i].sval; i++) {
+		if (strcmp(sval, xlatstr[i].sval) == 0) {
+		    val = xlatstr[i].ival;
+		    goto found;
+		}
+	    }
+	    return GE_INVAL;
+	} else {
+	    val = strtol(sval, NULL, 0);
+	}
+    }
+ found:
+    if (done || cdone) {
 	qe = sdata->o->zalloc(sdata->o, sizeof(*qe));
 	if (!qe)
 	    return GE_NOMEM;
 	qe->xlat = xlat;
+	qe->cdone = cdone;
+	qe->xlatstr = xlatstr;
 	qe->done = done;
 	qe->cb_data = cb_data;
 	qe->op = op;
@@ -341,47 +399,62 @@ serconf_set_get(struct sterm_data *sdata, int op, int val,
 }
 
 static int
-sterm_baud(struct sergensio *sio, int baud,
+sterm_baud(struct sterm_data *sdata, int baud, const char *sbaud,
+	   gensio_control_done cdone,
 	   void (*done)(struct sergensio *sio, int err,
 			int baud, void *cb_data),
 	   void *cb_data)
 {
-    return serconf_set_get(sergensio_get_gensio_data(sio),
-			   GENSIO_IOD_CONTROL_BAUD, baud,
-			   NULL, done, cb_data);
+    return serconf_set_get(sdata,
+			   GENSIO_IOD_CONTROL_BAUD, baud, sbaud,
+			   NULL, cdone, NULL, done, cb_data);
 }
 
 static int
-sterm_datasize(struct sergensio *sio, int datasize,
+sterm_datasize(struct sterm_data *sdata, int datasize, const char *sdatasize,
+	       gensio_control_done cdone,
 	       void (*done)(struct sergensio *sio, int err, int datasize,
 			    void *cb_data),
 	       void *cb_data)
 {
-    return serconf_set_get(sergensio_get_gensio_data(sio),
-			   GENSIO_IOD_CONTROL_DATASIZE, datasize,
-			   NULL, done, cb_data);
+    return serconf_set_get(sdata,
+			   GENSIO_IOD_CONTROL_DATASIZE, datasize, sdatasize,
+			   NULL, cdone, NULL, done, cb_data);
 }
 
+static const struct termio_xlat_str sterm_parity_xlatstr[] = {
+    { "0", 0 },
+    { "", 0 },
+    { "none", SERGENSIO_PARITY_NONE },
+    { "odd", SERGENSIO_PARITY_ODD },
+    { "even", SERGENSIO_PARITY_EVEN },
+    { "mark", SERGENSIO_PARITY_MARK },
+    { "space", SERGENSIO_PARITY_SPACE },
+    {}
+};
+
 static int
-sterm_parity(struct sergensio *sio, int parity,
+sterm_parity(struct sterm_data *sdata, int parity, const char *sparity,
+	     gensio_control_done cdone,
 	     void (*done)(struct sergensio *sio, int err, int parity,
 			  void *cb_data),
 	     void *cb_data)
 {
-    return serconf_set_get(sergensio_get_gensio_data(sio),
-			   GENSIO_IOD_CONTROL_PARITY, parity,
-			   NULL, done, cb_data);
+    return serconf_set_get(sdata,
+			   GENSIO_IOD_CONTROL_PARITY, parity, sparity,
+			   NULL, cdone, sterm_parity_xlatstr, done, cb_data);
 }
 
 static int
-sterm_stopbits(struct sergensio *sio, int stopbits,
+sterm_stopbits(struct sterm_data *sdata, int stopbits, const char *sstopbits,
+	       gensio_control_done cdone,
 	       void (*done)(struct sergensio *sio, int err, int stopbits,
 			    void *cb_data),
 	       void *cb_data)
 {
-    return serconf_set_get(sergensio_get_gensio_data(sio),
-			   GENSIO_IOD_CONTROL_STOPBITS, stopbits,
-			   NULL, done, cb_data);
+    return serconf_set_get(sdata,
+			   GENSIO_IOD_CONTROL_STOPBITS, stopbits, sstopbits,
+			   NULL, cdone, NULL, done, cb_data);
 }
 
 static int
@@ -449,14 +522,37 @@ serconf_xlat_flowcontrol(struct sterm_data *sdata, bool get,
     return 0;
 }
 
+static const struct termio_xlat_str sterm_flow_xlatstr[] = {
+    { "0", 0 },
+    { "", 0 },
+    { "none", SERGENSIO_FLOWCONTROL_NONE },
+    { "xonxoff", SERGENSIO_FLOWCONTROL_XON_XOFF },
+    { "rtscts", SERGENSIO_FLOWCONTROL_RTS_CTS },
+    {}
+};
+
 static int
-sterm_flowcontrol(struct sergensio *sio, int flowcontrol,
+sterm_flowcontrol(struct sterm_data *sdata, int flowcontrol,
+		  const char *sflowcontrol,
+		  gensio_control_done cdone,
 		  void (*done)(struct sergensio *sio, int err,
 			       int flowcontrol, void *cb_data),
 		  void *cb_data)
 {
-    struct sterm_data *sdata = sergensio_get_gensio_data(sio);
+    const struct termio_xlat_str *xlatstr = sterm_flow_xlatstr;
 
+    if (sflowcontrol) {
+	unsigned int i;
+
+	for (i = 0; xlatstr[i].sval; i++) {
+	    if (strcmp(sflowcontrol, xlatstr[i].sval) == 0) {
+		flowcontrol = xlatstr[i].ival;
+		goto found;
+	    }
+	}
+	return GE_INVAL;
+    }
+ found:
     switch (flowcontrol) {
     case SERGENSIO_FLOWCONTROL_NONE:
     case SERGENSIO_FLOWCONTROL_RTS_CTS:
@@ -470,21 +566,43 @@ sterm_flowcontrol(struct sergensio *sio, int flowcontrol,
     }
 
     return serconf_set_get(sdata, GENSIO_IOD_CONTROL_RTSCTS, flowcontrol,
-			   serconf_xlat_flowcontrol, done, cb_data);
+			   NULL, serconf_xlat_flowcontrol, cdone, xlatstr,
+			   done, cb_data);
 }
 
+static const struct termio_xlat_str sterm_iflow_xlatstr[] = {
+    { "0", 0 },
+    { "", 0 },
+    { "none", SERGENSIO_FLOWCONTROL_NONE },
+    { "dcd", SERGENSIO_FLOWCONTROL_DCD },
+    { "dtr", SERGENSIO_FLOWCONTROL_DTR },
+    { "dsr", SERGENSIO_FLOWCONTROL_DSR },
+    {}
+};
+
 static int
-sterm_iflowcontrol(struct sergensio *sio, int iflowcontrol,
+sterm_iflowcontrol(struct sterm_data *sdata, int iflowcontrol,
+		   const char *siflowcontrol,
+		   gensio_control_done cdone,
 		   void (*done)(struct sergensio *sio, int err,
 				int iflowcontrol, void *cb_data),
 		   void *cb_data)
 {
     /* Input flow control is not independently settable. */
-    return serconf_set_get(sergensio_get_gensio_data(sio),
-			   GENSIO_IOD_CONTROL_XONXOFF, 0,
-			   serconf_xlat_flowcontrol, done, cb_data);
+    return serconf_set_get(sdata,
+			   GENSIO_IOD_CONTROL_XONXOFF, 0, siflowcontrol,
+			   serconf_xlat_flowcontrol,
+			   cdone, sterm_iflow_xlatstr, done, cb_data);
 }
 
+static const struct termio_xlat_str sterm_on_off_xlatstr[] = {
+    { "0", 0 },
+    { "", 0 },
+    { "on", GENSIO_SER_ON },
+    { "off", GENSIO_SER_OFF },
+    {}
+};
+
 static int
 sterm_xlat_sbreak(struct sterm_data *sdata, bool get, int *oval, int val)
 {
@@ -506,14 +624,16 @@ sterm_xlat_sbreak(struct sterm_data *sdata, bool get, int *oval, int val)
 }
 
 static int
-sterm_sbreak(struct sergensio *sio, int breakv,
+sterm_sbreak(struct sterm_data *sdata, int breakv, const char *sbreakv,
+	     gensio_control_done cdone,
 	     void (*done)(struct sergensio *sio, int err, int breakv,
 			  void *cb_data),
 	     void *cb_data)
 {
-    return serconf_set_get(sergensio_get_gensio_data(sio),
-			   GENSIO_IOD_CONTROL_SET_BREAK, breakv,
-			   sterm_xlat_sbreak, done, cb_data);
+    return serconf_set_get(sdata,
+			   GENSIO_IOD_CONTROL_SET_BREAK, breakv, sbreakv,
+			   sterm_xlat_sbreak, cdone, sterm_on_off_xlatstr,
+			   done, cb_data);
 }
 
 static int
@@ -537,14 +657,16 @@ serconf_xlat_dtr(struct sterm_data *sdata, bool get, int *oval, int val)
 }
 
 static int
-sterm_dtr(struct sergensio *sio, int dtr,
+sterm_dtr(struct sterm_data *sdata, int dtr, const char *sdtr,
+	  gensio_control_done cdone,
 	  void (*done)(struct sergensio *sio, int err, int dtr,
 		       void *cb_data),
 	  void *cb_data)
 {
-    return serconf_set_get(sergensio_get_gensio_data(sio),
-			   GENSIO_IOD_CONTROL_DTR, dtr,
-			   serconf_xlat_dtr, done, cb_data);
+    return serconf_set_get(sdata,
+			   GENSIO_IOD_CONTROL_DTR, dtr, sdtr,
+			   serconf_xlat_dtr, cdone, sterm_on_off_xlatstr,
+			   done, cb_data);
 }
 
 static int
@@ -568,14 +690,16 @@ serconf_xlat_rts(struct sterm_data *sdata, bool get, int *oval, int val)
 }
 
 static int
-sterm_rts(struct sergensio *sio, int rts,
+sterm_rts(struct sterm_data *sdata, int rts, const char *srts,
+	  gensio_control_done cdone,
 	  void (*done)(struct sergensio *sio, int err, int rts,
 		       void *cb_data),
 	  void *cb_data)
 {
-    return serconf_set_get(sergensio_get_gensio_data(sio),
-			   GENSIO_IOD_CONTROL_RTS, rts,
-			   serconf_xlat_rts, done, cb_data);
+    return serconf_set_get(sdata,
+			   GENSIO_IOD_CONTROL_RTS, rts, srts,
+			   serconf_xlat_rts, cdone, sterm_on_off_xlatstr,
+			   done, cb_data);
 }
 
 static void
@@ -631,11 +755,13 @@ serialdev_timeout(struct gensio_timer *t, void *cb_data)
 }
 
 static int
-sterm_modemstate(struct sergensio *sio, unsigned int val)
+sterm_modemstate(struct sterm_data *sdata, unsigned int val, const char *sval)
 {
-    struct sterm_data *sdata = sergensio_get_gensio_data(sio);
     gensio_time timeout = {0, 0};
 
+    if (sval)
+	val = strtol(sval, NULL, 0);
+
     sterm_lock(sdata);
     sdata->modemstate_mask = val;
     sdata->sent_first_modemstate = false;
@@ -648,21 +774,38 @@ sterm_modemstate(struct sergensio *sio, unsigned int val)
 }
 
 static int
-sterm_flowcontrol_state(struct sergensio *sio, bool val)
+sterm_flowcontrol_state(struct sterm_data *sdata, bool val, char *sval)
 {
-    struct sterm_data *sdata = sergensio_get_gensio_data(sio);
+    if (sval) {
+	if (strcmp(sval, "true") == 0 || strcmp(sval, "on") == 0)
+	    val = true;
+	else if (strcmp(sval, "false") == 0 || strcmp(sval, "off") == 0)
+	    val = false;
+	else
+	    val = strtol(sval, NULL, 0);
+    }
 
     return sdata->o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_FLOWCTL_STATE,
 				 false, val);
 }
 
 static int
-sterm_flush(struct sergensio *sio, unsigned int val)
+sterm_flush(struct sterm_data *sdata, unsigned int val, const char *sval)
 {
-    struct sterm_data *sdata = sergensio_get_gensio_data(sio);
     struct gensio_os_funcs *o = sdata->o;
     int tval;
 
+    if (sval) {
+	if (strcmp(sval, "recv") == 0)
+	    val = 1;
+	else if (strcmp(sval, "xmit") == 0)
+	    val = 2;
+	else if (strcmp(sval, "both") == 0)
+	    val = 3;
+	else
+	    return GE_INVAL;
+    }
+
     switch(val) {
     case SERGENSIO_FLUSH_RCV_BUFFER:	tval = GENSIO_IN_BUF; break;
     case SERGENSIO_FLUSH_XMIT_BUFFER:	tval = GENSIO_OUT_BUF; break;
@@ -678,10 +821,8 @@ sterm_flush(struct sergensio *sio, unsigned int val)
 }
 
 static int
-sterm_send_break(struct sergensio *sio)
+sterm_send_break(struct sterm_data *sdata)
 {
-    struct sterm_data *sdata = sergensio_get_gensio_data(sio);
-
     return sdata->o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_SEND_BREAK,
 				 false, 0);
 }
@@ -692,48 +833,56 @@ sergensio_sterm_func(struct sergensio *sio, int op, int val, char *buf,
 {
     struct sterm_data *sdata = sergensio_get_gensio_data(sio);
 
-    if (sdata->write_only)
+    if (!sdata->set_tty)
 	return GE_NOTSUP;
 
     switch (op) {
     case SERGENSIO_FUNC_BAUD:
-	return sterm_baud(sio, val, done, cb_data);
+	return sterm_baud(sdata, val, NULL, NULL, done, cb_data);
 
     case SERGENSIO_FUNC_DATASIZE:
-	return sterm_datasize(sio, val, done, cb_data);
+	return sterm_datasize(sdata, val, NULL, NULL, done, cb_data);
 
     case SERGENSIO_FUNC_PARITY:
-	return sterm_parity(sio, val, done, cb_data);
+	return sterm_parity(sdata, val, NULL, NULL, done, cb_data);
 
     case SERGENSIO_FUNC_STOPBITS:
-	return sterm_stopbits(sio, val, done, cb_data);
+	return sterm_stopbits(sdata, val, NULL, NULL, done, cb_data);
 
     case SERGENSIO_FUNC_FLOWCONTROL:
-	return sterm_flowcontrol(sio, val, done, cb_data);
+	return sterm_flowcontrol(sdata, val, NULL, NULL, done, cb_data);
 
     case SERGENSIO_FUNC_IFLOWCONTROL:
-	return sterm_iflowcontrol(sio, val, done, cb_data);
+	return sterm_iflowcontrol(sdata, val, NULL, NULL, done, cb_data);
 
     case SERGENSIO_FUNC_SBREAK:
-	return sterm_sbreak(sio, val, done, cb_data);
+	return sterm_sbreak(sdata, val, NULL, NULL, done, cb_data);
 
     case SERGENSIO_FUNC_DTR:
-	return sterm_dtr(sio, val, done, cb_data);
+	return sterm_dtr(sdata, val, NULL, NULL, done, cb_data);
 
     case SERGENSIO_FUNC_RTS:
-	return sterm_rts(sio, val, done, cb_data);
+	return sterm_rts(sdata, val, NULL, NULL, done, cb_data);
 
     case SERGENSIO_FUNC_MODEMSTATE:
-	return sterm_modemstate(sio, val);
+	if (done)
+	    return GE_INVAL;
+	return sterm_modemstate(sdata, val, NULL);
 
     case SERGENSIO_FUNC_FLOWCONTROL_STATE:
-	return sterm_flowcontrol_state(sio, val);
+	if (done)
+	    return GE_INVAL;
+	return sterm_flowcontrol_state(sdata, val, NULL);
 
     case SERGENSIO_FUNC_FLUSH:
-	return sterm_flush(sio, val);
+	if (done)
+	    return GE_INVAL;
+	return sterm_flush(sdata, val, NULL);
 
     case SERGENSIO_FUNC_SEND_BREAK:
-	return sterm_send_break(sio);
+	if (done)
+	    return GE_INVAL;
+	return sterm_send_break(sdata);
 
     case SERGENSIO_FUNC_SIGNATURE:
     case SERGENSIO_FUNC_LINESTATE:
@@ -742,6 +891,55 @@ sergensio_sterm_func(struct sergensio *sio, int op, int val, char *buf,
     }
 }
 
+static int
+sterm_acontrol(void *handler_data, struct gensio_iod *iod, bool get,
+	       unsigned int option,
+	       struct gensio_func_acontrol *idata)
+{
+    struct sterm_data *sdata = handler_data;
+    const char *data = NULL;
+    gensio_control_done done = idata->done;
+    void *cb_data = idata->cb_data;;
+
+    if (!sdata->set_tty)
+	return GE_NOTSUP;
+
+    if (!get) /* On a get, set val to 0 and data to NULL to fetch the value. */
+	data = idata->data;
+
+    switch (option) {
+    case GENSIO_ACONTROL_SER_BAUD:
+	return sterm_baud(sdata, 0, data, done, NULL, cb_data);
+
+    case GENSIO_ACONTROL_SER_DATASIZE:
+	return sterm_datasize(sdata, 0, data, done, NULL, cb_data);
+
+    case GENSIO_ACONTROL_SER_PARITY:
+	return sterm_parity(sdata, 0, data, done, NULL, cb_data);
+
+    case GENSIO_ACONTROL_SER_STOPBITS:
+	return sterm_stopbits(sdata, 0, data, done, NULL, cb_data);
+
+    case GENSIO_ACONTROL_SER_FLOWCONTROL:
+	return sterm_flowcontrol(sdata, 0, data, done, NULL, cb_data);
+
+    case GENSIO_ACONTROL_SER_IFLOWCONTROL:
+	return sterm_iflowcontrol(sdata, 0, data, done, NULL, cb_data);
+
+    case GENSIO_ACONTROL_SER_SBREAK:
+	return sterm_sbreak(sdata, 0, data, done, NULL, cb_data);
+
+    case GENSIO_ACONTROL_SER_DTR:
+	return sterm_dtr(sdata, 0, data, done, NULL, cb_data);
+
+    case GENSIO_ACONTROL_SER_RTS:
+	return sterm_rts(sdata, 0, data, done, NULL, cb_data);
+
+    default:
+	return GE_NOTSUP;
+    }
+}
+
 static void
 sterm_timer_stopped(struct gensio_timer *timer, void *cb_data)
 {
@@ -786,13 +984,13 @@ sterm_check_close_drain(void *handler_data, struct gensio_iod *iod,
 
     rv = o->bufcount(sdata->iod, GENSIO_OUT_BUF, &count);
     if (rv || count <= 0)
-	goto out_rm_uucp;
+	goto out_rm_lock;
     if (sdata->last_close_outq_count == 0)
 	/* First time through, set the total time. */
 	sdata->close_timeouts_left = sdata->drain_time;
     if (sdata->close_timeouts_left >= 0) {
 	if (sdata->close_timeouts_left == 0)
-	    goto out_rm_uucp;
+	    goto out_rm_lock;
 	sdata->close_timeouts_left--;
     }
 
@@ -805,7 +1003,7 @@ sterm_check_close_drain(void *handler_data, struct gensio_iod *iod,
 
     if (sdata->char_timeouts_left >= 0) {
 	if (sdata->char_timeouts_left == 0)
-	    goto out_rm_uucp;
+	    goto out_rm_lock;
 	sdata->char_timeouts_left--;
     }
 
@@ -813,11 +1011,11 @@ sterm_check_close_drain(void *handler_data, struct gensio_iod *iod,
     err = GE_INPROGRESS;
     next_timeout->secs = 0;
     next_timeout->nsecs = 10000000;
- out_rm_uucp:
+ out_rm_lock:
     if (!err) {
 	o->flush(sdata->iod, GENSIO_OUT_BUF);
-	if (!sdata->no_uucp_lock)
-	    uucp_rm_lock(sdata->devname);
+	serial_rm_lock(o, sdata->ll, sdata->uucp_lock, sdata->flock_lock,
+		       o->iod_get_fd(sdata->iod), sdata->devname);
 	gensio_fd_ll_close_now(sdata->ll);
     }
     if (err != GE_INPROGRESS)
@@ -867,104 +1065,107 @@ sterm_sub_open(void *handler_data, struct gensio_iod **riod)
     struct sterm_data *sdata = handler_data;
     struct gensio_os_funcs *o = sdata->o;
     int err;
-    int options;
-
-    if (!sdata->no_uucp_lock) {
-	err = uucp_mk_lock(o, sdata->devname);
-	if (err)
-	    goto out;
-    }
+    int options = 0;
 
     sdata->timer_stopped = false;
     sdata->iod = NULL; /* If it's a re-open make sure this is clear. */
 
-    options = GENSIO_OPEN_OPTION_WRITEABLE;
+    if (!sdata->read_only)
+	options |= GENSIO_OPEN_OPTION_WRITEABLE;
     if (!sdata->write_only)
 	options |= GENSIO_OPEN_OPTION_READABLE;
     err = o->open_dev(o, sdata->devname, options, &sdata->iod);
     if (err)
-	goto out_uucp;
+	goto out;
 
-    if (!sdata->write_only) {
+    err = serial_mk_lock(o, sdata->ll, sdata->uucp_lock, sdata->flock_lock,
+			 o->iod_get_fd(sdata->iod), sdata->devname);
+    if (err)
+	goto out;
+
+    if (sdata->set_tty) {
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_BAUD, false,
 			     sdata->def_baud);
 	if (err)
-	    goto out_uucp;
+	    goto out_unlock;
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_PARITY, false,
 			     sdata->def_parity);
 	if (err)
-	    goto out_uucp;
+	    goto out_unlock;
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_XONXOFF, false,
 			     sdata->def_xonxoff);
 	if (err)
-	    goto out_uucp;
+	    goto out_unlock;
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_IXONXOFF, false,
 			     sdata->def_xonxoff);
 	if (err)
-	    goto out_uucp;
+	    goto out_unlock;
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_RTSCTS, false,
 			     sdata->def_rtscts);
 	if (err)
-	    goto out_uucp;
+	    goto out_unlock;
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_DATASIZE, false,
 			     sdata->def_datasize);
 	if (err)
-	    goto out_uucp;
+	    goto out_unlock;
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_STOPBITS, false,
 			     sdata->def_stopbits);
 	if (err)
-	    goto out_uucp;
+	    goto out_unlock;
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_LOCAL, false,
 			     sdata->def_local);
 	if (err)
-	    goto out_uucp;
-	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_HANGUP_ON_DONE,
-			     false, sdata->def_hupcl);
-	if (err)
-	    goto out_uucp;
+	    goto out_unlock;
+	if (sdata->def_hupcl >= 0) {
+	    err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_HANGUP_ON_DONE,
+				 false, sdata->def_hupcl);
+	    if (err)
+		goto out_unlock;
+	}
 	if (sdata->rs485) {
 	    err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_RS485, false,
 				 (intptr_t) sdata->rs485);
 	    if (err)
-		goto out_uucp;
+		goto out_unlock;
 	}
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_APPLY, false, 0);
 	if (err)
-	    goto out_uucp;
+	    goto out_unlock;
 	if (sdata->rts_set && sdata->rts_first) {
 	    err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_RTS, false,
 				 sdata->rts_val);
 	    if (err)
-		goto out_uucp;
+		goto out_unlock;
 	}
 	if (sdata->dtr_set) {
 	    err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_DTR, false,
 				 sdata->dtr_val);
 	    if (err)
-		goto out_uucp;
+		goto out_unlock;
 	}
 	if (sdata->rts_set && !sdata->rts_first) {
 	    err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_RTS, false,
 				 sdata->rts_val);
 	    if (err)
-		goto out_uucp;
+		goto out_unlock;
 	}
     }
 
-    if (!sdata->write_only && !sdata->disablebreak) {
+    if (sdata->set_tty && !sdata->disablebreak) {
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_SET_BREAK,
 			     false, sdata->disablebreak);
 	if (err)
-	    gensio_log(o, GENSIO_LOG_WARNING,
-		       "serialdev: "
-		       "Setting break failed on %s, "
-		       "try adding the nobreak option: %s",
-		       sdata->devname, gensio_err_to_str(err));
+	    gensio_ll_log(sdata->ll, GENSIO_LOG_WARNING,
+			  "serialdev: "
+			  "Setting break failed on %s, "
+			  "try adding the nobreak option: %s",
+			  sdata->devname, gensio_err_to_str(err));
 	/*
-	 * Do not fail on an error here.  There are USB and bluetooth devices that fail
-	 * this because they don't implement it, but it's impossible to tell beforehand
-	 * that they will fail.  This shouldn't be able to fail on a working device, so
-	 * just log and allow it.
+	 * Do not fail on an error here.  There are USB and bluetooth
+	 * devices that fail this because they don't implement it, but
+	 * it's impossible to tell beforehand that they will fail.
+	 * This shouldn't be able to fail on a working device, so just
+	 * log and allow it.
 	 */
     }
 
@@ -973,16 +1174,16 @@ sterm_sub_open(void *handler_data, struct gensio_iod **riod)
     sdata->sent_first_modemstate = false;
     sterm_unlock(sdata);
 
-    if (!sdata->write_only)
-	sterm_modemstate(sdata->sio, 255);
+    if (sdata->set_tty)
+	sterm_modemstate(sdata, 255, NULL);
 
     *riod = sdata->iod;
 
     return 0;
 
- out_uucp:
-    if (!sdata->no_uucp_lock)
-	uucp_rm_lock(sdata->devname);
+ out_unlock:
+    serial_rm_lock(o, sdata->ll, sdata->uucp_lock, sdata->flock_lock,
+		   o->iod_get_fd(sdata->iod), sdata->devname);
 
     /* pty's for some reason return EIO if the remote end closes. */
     if (sdata->is_pty && err == GE_IOERR)
@@ -1025,7 +1226,7 @@ sterm_control_raddr(struct sterm_data *sdata, char *buf, gensiods *datalen)
 
     gensio_pos_snprintf(buf, buflen, &pos, "%s", sdata->devname);
 
-    if (!sdata->write_only) {
+    if (sdata->set_tty) {
 	int baud;
 	int stopbits;
 	int datasize;
@@ -1102,7 +1303,7 @@ sterm_control_raddr(struct sterm_data *sdata, char *buf, gensiods *datalen)
 	    gensio_pos_snprintf(buf, buflen, &pos, ",HANGUP_WHEN_DONE");
 
     }
-    if (!sdata->write_only && sdata->iod) {
+    if (sdata->set_tty && sdata->iod) {
 	rv = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_RTS,
 			    true, (intptr_t) &tval);
 	if (rv)
@@ -1137,13 +1338,13 @@ sterm_control(void *handler_data, struct gensio_iod *iod,
     switch (option) {
     case GENSIO_CONTROL_SEND_BREAK:
 	if (get)
-	    break;
+	    return GE_NOTSUP;
 	return sdata->o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_SEND_BREAK,
 				     false, 0);
 
     case GENSIO_CONTROL_RADDR:
 	if (!get)
-	    break;
+	    return GE_NOTSUP;
 	if (strtoul(data, NULL, 0) > 0)
 	    return GE_NOTFOUND;
 	return sterm_control_raddr(sdata, data, datalen);
@@ -1155,11 +1356,29 @@ sterm_control(void *handler_data, struct gensio_iod *iod,
 			    sdata->o->iod_get_fd(sdata->iod));
 	return 0;
 
-    default:
-	break;
-    }
+    case GENSIO_CONTROL_SER_MODEMSTATE:
+	if (get)
+	    return GE_NOTSUP;
+	return sterm_modemstate(sdata, 0, data);
 
-    return GE_NOTSUP;
+    case GENSIO_CONTROL_SER_FLOWCONTROL_STATE:
+	if (get)
+	    return GE_NOTSUP;
+	return sterm_flowcontrol_state(sdata, 0, data);
+
+    case GENSIO_CONTROL_SER_FLUSH:
+	if (get)
+	    return GE_NOTSUP;
+	return sterm_flush(sdata, 0, data);
+
+    case GENSIO_CONTROL_SER_SEND_BREAK:
+	if (get)
+	    return GE_NOTSUP;
+	return sterm_send_break(sdata);
+
+    default:
+	return GE_NOTSUP;
+    }
 }
 
 static int
@@ -1201,7 +1420,8 @@ static const struct gensio_fd_ll_ops sterm_fd_ll_ops = {
     .free = sterm_free,
     .write = sterm_write,
     .read_ready = sterm_read_ready,
-    .control = sterm_control
+    .control = sterm_control,
+    .acontrol = sterm_acontrol
 };
 
 static int
@@ -1301,6 +1521,8 @@ sergensio_process_parms(struct gensio_pparm_info *p, struct sterm_data *sdata)
 
     for (i = 0; i < argc; i++) {
 	if (gensio_pparm_bool(p, argv[i], "wronly", &sdata->write_only) > 0) {
+	    /* In the later parms, wronly sets both write only and set_tty. */
+	    sdata->set_tty = !sdata->write_only;
 	    continue;
 	} else if (gensio_pparm_bool(p, argv[i], "nobreak",
 				     &sdata->disablebreak) > 0) {
@@ -1379,8 +1601,8 @@ sergensio_setup_defaults(struct gensio_pparm_info *p,
     sdata->def_local = val;
 
     val = 0;
-    err = gensio_get_default(o, "serialdev", "hangup_when_done", false,
-			     GENSIO_DEFAULT_BOOL, NULL, &val);
+    err = gensio_get_default(o, "serialdev", "hangup-when-done", false,
+			     GENSIO_DEFAULT_INT, NULL, &val);
     if (err)
 	return err;
     sdata->def_hupcl = val;
@@ -1398,19 +1620,18 @@ sergensio_setup_defaults(struct gensio_pparm_info *p,
 }
 
 static int
-serialdev_gensio_alloc(const void *gdata, const char * const args[],
-		       struct gensio_os_funcs *o,
-		       gensio_event cb, void *user_data,
-		       struct gensio **rio)
+iodev_gensio_alloc(const void *gdata, const char * const args[],
+		   struct gensio_os_funcs *o, bool set_tty,
+		   gensio_event cb, void *user_data,
+		   struct gensio **rio)
 {
     const char *devname = gdata;
     struct sterm_data *sdata = o->zalloc(o, sizeof(*sdata));
-    struct gensio *io;
     int err;
     char *comma;
     gensiods max_read_size = GENSIO_DEFAULT_BUF_SIZE;
     int i, ival;
-    bool nouucplock_set = false, dummy = false;
+    bool lock_set = false, dummy = false, wronly = false, rdonly = false;
     const char *s;
     char *end;
     GENSIO_DECLARE_PPGENSIO(p, o, cb, "serialdev", user_data);
@@ -1420,11 +1641,21 @@ serialdev_gensio_alloc(const void *gdata, const char * const args[],
 
     sdata->o = o;
 
-    err = gensio_get_default(o, "sergensio", "nouucplock", false,
-			     GENSIO_DEFAULT_BOOL, NULL, &ival);
-    if (err)
-	goto out_err;
-    sdata->no_uucp_lock = ival;
+    if (!set_tty) {
+	sdata->uucp_lock = false;
+	sdata->flock_lock = false;
+    } else {
+	err = gensio_get_default(o, "sergensio", "uucplock", false,
+				 GENSIO_DEFAULT_BOOL, NULL, &ival);
+	if (err)
+	    goto out_err;
+	sdata->uucp_lock = ival;
+	err = gensio_get_default(o, "sergensio", "flock", false,
+				 GENSIO_DEFAULT_BOOL, NULL, &ival);
+	if (err)
+	    goto out_err;
+	sdata->flock_lock = ival;
+    }
     err = gensio_get_default(o, "sergensio", "drain_time", false,
 			     GENSIO_DEFAULT_INT, NULL, &sdata->drain_time);
     if (err)
@@ -1437,6 +1668,10 @@ serialdev_gensio_alloc(const void *gdata, const char * const args[],
     for (i = 0; args && args[i]; i++) {
 	if (gensio_pparm_ds(&p, args[i], "readbuf", &max_read_size) > 0)
 	    continue;
+	if (gensio_pparm_bool(&p, args[i], "wronly", &wronly) > 0)
+	    continue;
+	if (gensio_pparm_bool(&p, args[i], "rdonly", &rdonly) > 0)
+	    continue;
 	if (gensio_pparm_value(&p, args[i], "drain_time", &s) > 0) {
 	    if (strcmp(s, "off") == 0) {
 		sdata->drain_time = -1;
@@ -1457,9 +1692,20 @@ serialdev_gensio_alloc(const void *gdata, const char * const args[],
 	    }
 	    continue;
 	}
-	if (gensio_pparm_bool(&p, args[i], "nouucplock",
-			      &sdata->no_uucp_lock) > 0) {
-	    nouucplock_set = true;
+	if (set_tty && gensio_pparm_bool(&p, args[i], "nouucplock",
+			      &sdata->uucp_lock) > 0) {
+	    sdata->uucp_lock = !sdata->uucp_lock;
+	    lock_set = true;
+	    continue;
+	}
+	if (set_tty && gensio_pparm_bool(&p, args[i], "uucplock",
+			      &sdata->uucp_lock) > 0) {
+	    lock_set = true;
+	    continue;
+	}
+	if (set_tty && gensio_pparm_bool(&p, args[i], "flock",
+					 &sdata->flock_lock) > 0) {
+	    lock_set = true;
 	    continue;
 	}
 	/* custspeed is ignored now */
@@ -1471,6 +1717,12 @@ serialdev_gensio_alloc(const void *gdata, const char * const args[],
 	goto out_err;
     }
 
+    if (wronly && rdonly) {
+	gensio_pparm_slog(&p, "You cannot set both wronly and rdonly");
+	err = GE_INVAL;
+	goto out_err;
+    }
+
     sdata->timer = o->alloc_timer(o, serialdev_timeout, sdata);
     if (!sdata->timer)
 	goto out_nomem;
@@ -1480,16 +1732,19 @@ serialdev_gensio_alloc(const void *gdata, const char * const args[],
 	goto out_nomem;
 
     sdata->is_pty = is_a_pty(sdata->devname);
+    sdata->write_only = wronly;
+    sdata->read_only = rdonly;
+    sdata->set_tty = set_tty;
 
     comma = strchr(sdata->devname, ',');
     if (comma)
 	*comma++ = '\0';
 
-    if (!nouucplock_set) {
+    if (set_tty && !lock_set) {
 	const char *slash = strrchr(devname, '/');
 
 	/*
-	 * If the user didn't force it, don't do uucp locking if the
+	 * If the user didn't force it, don't do locking if the
 	 * devname is "tty", as in "/dev/tty".  That does all sorts
 	 * of bad things...
 	 */
@@ -1498,20 +1753,29 @@ serialdev_gensio_alloc(const void *gdata, const char * const args[],
 	else
 	    slash = devname;
 
-	/* Don't do uucp locking on /dev/tty or ptys */
-	sdata->no_uucp_lock = strcmp(slash, "tty") == 0 || sdata->is_pty;
+	/* Don't do uucp locking on /dev/tty or ptys.  flock is on on ptys */
+	sdata->uucp_lock = !(strcmp(slash, "tty") == 0 || sdata->is_pty);
+	sdata->flock_lock = !(strcmp(slash, "tty") == 0);
     }
 
-    err = sergensio_setup_defaults(&p, sdata);
-    if (err)
-	goto out_err;
-
-    if (comma) {
-	sdata->parms = comma;
-	err = sergensio_process_parms(&p, sdata);
+    if (set_tty) {
+	err = sergensio_setup_defaults(&p, sdata);
 	if (err)
 	    goto out_err;
+
+	if (comma) {
+	    sdata->parms = comma;
+	    err = sergensio_process_parms(&p, sdata);
+	    if (err)
+		goto out_err;
+	}
+    } else if (comma) {
+	gensio_pparm_slog(&p,
+			  "Serial port options not accepted for 'dev' gensios");
+	err = GE_INVAL;
+	goto out_err;
     }
+
     sdata->deferred_op_runner = o->alloc_runner(o, sterm_deferred_op, sdata);
     if (!sdata->deferred_op_runner)
 	goto out_nomem;
@@ -1521,7 +1785,8 @@ serialdev_gensio_alloc(const void *gdata, const char * const args[],
 	goto out_nomem;
 
     sdata->ll = fd_gensio_ll_alloc(o, NULL, &sterm_fd_ll_ops, sdata,
-				   max_read_size, sdata->write_only);
+				   max_read_size, sdata->write_only,
+				   sdata->read_only);
     if (!sdata->ll)
 	goto out_nomem;
 
@@ -1530,20 +1795,23 @@ serialdev_gensio_alloc(const void *gdata, const char * const args[],
      * the free callbacks.
      */
 
-    io = base_gensio_alloc(o, sdata->ll, NULL, NULL, "serialdev",
-			   cb, user_data);
-    if (!io) {
+    sdata->io = base_gensio_alloc(o, sdata->ll, NULL, NULL, "serialdev",
+				  cb, user_data);
+    if (!sdata->io) {
 	gensio_ll_free(sdata->ll);
 	return GE_NOMEM;
     }
 
-    err = sergensio_addclass(o, io, sergensio_sterm_func, sdata, &sdata->sio);
+    gensio_set_is_serial(sdata->io, true);
+
+    err = sergensio_addclass(o, sdata->io, sergensio_sterm_func,
+			     sdata, &sdata->sio);
     if (err) {
-	gensio_free(io);
+	gensio_free(sdata->io);
 	return err;
     }
 
-    *rio = io;
+    *rio = sdata->io;
     return 0;
 
  out_nomem:
@@ -1553,6 +1821,15 @@ serialdev_gensio_alloc(const void *gdata, const char * const args[],
     return err;
 }
 
+static int
+serialdev_gensio_alloc(const void *gdata, const char * const args[],
+		       struct gensio_os_funcs *o,
+		       gensio_event cb, void *user_data,
+		       struct gensio **rio)
+{
+    return iodev_gensio_alloc(gdata, args, o, true, cb, user_data, rio);
+}
+
 static int
 str_to_serialdev_gensio(const char *str, const char * const args[],
 			struct gensio_os_funcs *o,
@@ -1562,6 +1839,24 @@ str_to_serialdev_gensio(const char *str, const char * const args[],
     return serialdev_gensio_alloc(str, args, o, cb, user_data, new_gensio);
 }
 
+static int
+dev_gensio_alloc(const void *gdata, const char * const args[],
+		 struct gensio_os_funcs *o,
+		 gensio_event cb, void *user_data,
+		 struct gensio **rio)
+{
+    return iodev_gensio_alloc(gdata, args, o, false, cb, user_data, rio);
+}
+
+static int
+str_to_dev_gensio(const char *str, const char * const args[],
+		  struct gensio_os_funcs *o,
+		  gensio_event cb, void *user_data,
+		  struct gensio **new_gensio)
+{
+    return dev_gensio_alloc(str, args, o, cb, user_data, new_gensio);
+}
+
 int
 gensio_init_serialdev(struct gensio_os_funcs *o)
 {
@@ -1571,5 +1866,16 @@ gensio_init_serialdev(struct gensio_os_funcs *o)
 			 serialdev_gensio_alloc);
     if (rv)
 	return rv;
+
+    rv = register_gensio(o, "sdev", str_to_serialdev_gensio,
+			 serialdev_gensio_alloc);
+    if (rv)
+	return rv;
+
+    rv = register_gensio(o, "dev", str_to_dev_gensio,
+			 dev_gensio_alloc);
+    if (rv)
+	return rv;
+
     return 0;
 }
