@@ -360,6 +360,31 @@ gensio_close_done(struct gensio *io, void *cb_data) {
     OI_PY_STATE_PUT(gstate);
 }
 
+static void
+gensio_control_cb(struct gensio *io, int err, const char *buf, gensiods len,
+		  void *cb_data) {
+    swig_cb_val *cb = (swig_cb_val *) cb_data;
+    swig_ref io_ref;
+    PyObject *args, *o;
+    OI_PY_STATE gstate;
+
+    gstate = OI_PY_STATE_GET();
+
+    io_ref = swig_make_ref(io, gensio);
+    args = PyTuple_New(3);
+    gensio_pyref(io);
+    PyTuple_SET_ITEM(args, 0, io_ref.val);
+    o = PyInt_FromLong(err);
+    PyTuple_SET_ITEM(args, 1, o);
+    o = PyBytes_FromStringAndSize(buf, len);
+    PyTuple_SET_ITEM(args, 2, o);
+
+    swig_finish_call(cb, "control_done", args, false);
+
+    deref_swig_cb_val(cb);
+    OI_PY_STATE_PUT(gstate);
+}
+
 static void
 sgensio_call(struct gensio *io, long val, const char *func)
 {
@@ -392,6 +417,12 @@ sgensio_modemstate(struct gensio *io, unsigned int modemstate)
     sgensio_call(io, modemstate, "modemstate");
 }
 
+static void
+sgensio_modemstate_mask(struct gensio *io, unsigned int modemstate)
+{
+    sgensio_call(io, modemstate, "modemstate_mask");
+}
+
 static void
 sgensio_linestate(struct gensio *io, unsigned int linestate)
 {
@@ -781,6 +812,8 @@ gensio_child_event(struct gensio *io, void *user_data, int event, int readerr,
 	size_t len;
 	char *str = NULL;
 
+	args = PyTuple_New(1);
+
 	va_copy(tmpva, p->args);
 	len = vsnprintf(str, 0, p->log, tmpva);
 	va_end(tmpva);
@@ -788,7 +821,6 @@ gensio_child_event(struct gensio *io, void *user_data, int event, int readerr,
 	if (!str)
 	    goto out_put;
 	vsnprintf(str, len + 1, p->log, p->args);
-	args = PyTuple_New(1);
 	o = OI_PI_FromString(str);
 	PyTuple_SET_ITEM(args, 0, o);
 	free(str);
@@ -816,10 +848,46 @@ gensio_child_event(struct gensio *io, void *user_data, int event, int readerr,
 
     }
 
+    case GENSIO_EVENT_LOG: {
+	struct gensio_log_data *p = (struct gensio_log_data *) buf;
+	va_list tmpva;
+	size_t len;
+	char *str = NULL;
+
+	args = PyTuple_New(2);
+
+	o = OI_PI_FromString(gensio_log_level_to_str(p->level));
+	PyTuple_SET_ITEM(args, 0, o);
+
+	va_copy(tmpva, p->args);
+	len = vsnprintf(str, 0, p->log, tmpva);
+	va_end(tmpva);
+	str = malloc(len + 1);
+	if (!str)
+	    goto out_put;
+	vsnprintf(str, len + 1, p->log, p->args);
+	o = OI_PI_FromString(str);
+	PyTuple_SET_ITEM(args, 1, o);
+	free(str);
+
+	if (PyObject_HasAttrString(data->handler_val, "log")) {
+	    o = swig_finish_call_rv(data->handler_val, "log", args, true);
+	    if (o)
+		Py_DECREF(o);
+	} else {
+	    rv = GE_NOTSUP;
+	}
+	break;
+    }
+
     case GENSIO_EVENT_SER_MODEMSTATE:
 	sgensio_modemstate(io, *((unsigned int *) buf));
 	break;
 
+    case GENSIO_EVENT_SER_MODEMSTATE_MASK:
+	sgensio_modemstate_mask(io, *((unsigned int *) buf));
+	break;
+
     case GENSIO_EVENT_SER_LINESTATE:
 	sgensio_linestate(io, *((unsigned int *) buf));
 	break;
@@ -1374,13 +1442,13 @@ static void gensio_mdns_cb(struct gensio_mdns_watch *watch,
 
     gstate = OI_PY_STATE_GET();
 
-    if (state == GENSIO_MDNS_ALL_FOR_NOW) {
+    if (state == GENSIO_MDNS_WATCH_ALL_FOR_NOW) {
 	swig_finish_call(w->cb_val, "mdns_all_for_now", NULL, true);
 	goto out;
     }
 
     args = PyTuple_New(9);
-    PyTuple_SET_ITEM(args, 0, PyBool_FromLong(state == GENSIO_MDNS_NEW_DATA));
+    PyTuple_SET_ITEM(args, 0, PyBool_FromLong(state == GENSIO_MDNS_WATCH_NEW_DATA));
     PyTuple_SET_ITEM(args, 1, PyInt_FromLong(ipinterface));
     PyTuple_SET_ITEM(args, 2, PyInt_FromLong(ipdomain));
     PyTuple_SET_ITEM(args, 3, OI_PI_FromStringN(name));
