@@ -346,8 +346,8 @@ class EventHandler:
         return
 
     def modemstate(self, modemstate):
-        """Called from a sergensio object when a modemstate monitor is enabled
-        and a modemstate value changes..
+        """Called from a sergensio client object when a modemstate monitor is
+        enabled and a modemstate value changes.
 
         modemstate -- A bitmask of SERGENSIO_MODEMSTATE_CTS,
             SERGENSIO_MODEMSTATE_DSR, SERGENSIO_MODEMSTATE_RI and
@@ -357,13 +357,24 @@ class EventHandler:
             SERGENSIO_MODEMSTATE_DSR_CHANGED,
             SERGENSIO_MODEMSTATE_RI_CHANGED, and
             SERGENSIO_MODEMSTATE_CD_CHANGED.
+
+        """
+
+    def modemstate_mask(self, modemstate):
+        """Called from a server sergensio object the user requests a change in
+        the modemstate mask.
+
+        modemstate -- A bitmask of SERGENSIO_MODEMSTATE_CTS,
+            SERGENSIO_MODEMSTATE_DSR, SERGENSIO_MODEMSTATE_RI and
+            SERGENSIO_MODEMSTATE_CD.  This is the current modemstate
+            mask.
         """
 
     def linestate(self, modemstate):
         """Called from a sergensio object when a linestate monitor is enabled
         and a particular linestate event occurs.
 
-        linesate -- A bitmask of
+        linestate -- A bitmask of
             SERGENSIO_LINESTATE_DATA_READY,
             SERGENSIO_LINESTATE_OVERRUN_ERR,
             SERGENSIO_LINESTATE_PARITY_ERR,
@@ -381,6 +392,17 @@ class EventHandler:
         log -- The detailed log of the parsing error.
         """
 
+    def win_size(self, height, width):
+        """Called when a window size change is reported"""
+
+    def log(self, level, log):
+        """Called to report internal logs on gensios, for debugging.
+
+        level -- A log level string: "fatal", "error", "warning",
+                "info", or "debug"
+        log -- The log text
+        """
+
 class OpenDone:
 
         """A template for a class handling the finish of an open."""
@@ -403,6 +425,19 @@ class CloseDone:
         """
         return
 
+class ControlDone:
+    """A template for a class handling the finish of a control operation."""
+
+    def control_done(self, io, err, val):
+        """Called when the close operation completes.
+
+        io -- The gensio the operation was against.
+        err -- A gensio error value.
+        val -- A string/bytestring with the result of the operation.  Will
+            not be set if err is not zero.
+        """
+        return
+
 class gensio:
     def __init__(o, gensiostr, handler):
         """Allocate a gensio.
@@ -619,6 +654,59 @@ class gensio:
         """
         return ""
 
+    def control_set(self, depth, option, data):
+        """Do a gensio-specific control set operation.  See the specific
+        gensios and the C interface for specific gensio controls.  This
+        is only for setting and returns an error value, not a string.
+
+        depth -- The gensio in the stack to choose.  0 selects the top
+            gensio, 1 the second from the top, etc.
+            GENSIO_CONTROL_DEPTH_ALL will call the control on every gensio
+            in the stack, GENSIO_CONTROL_DEPTH_FIRST will call the control
+            down the stack until a control doesn't return GE_NOTSOP.
+        option -- The specific gensio control to call.
+        data -- A string specifying the data for the control.
+
+        Returns a gensio error.  Does not raise an exception.
+        """
+        return ""
+
+    def acontrol(self, depth, get, option, data, done):
+        """Do a gensio-specific asynchronous control operation.  See the
+        specific gensios and the C interface for specific gensio
+        controls.  The operation is not complete when this returns, it
+        will be complete when the done callback is called.
+
+        depth -- The gensio in the stack to choose.  0 selects the top
+            gensio, 1 the second from the top, etc.
+            GENSIO_CONTROL_DEPTH_FIRST will call the control
+            down the stack until a control doesn't return GE_NOTSOP.
+        get -- A boolean specifying if this is a get or a put operation.
+        option -- The specific gensio control to call.
+        data -- A string specifying the data for the control.
+        done -- A ControlDone type callback called when the operation completes.
+
+        No return value.  This will raise an exception on an error.
+        """
+        return
+
+    def acontrol_s(self, depth, get, option, data):
+        """Do a gensio-specific asynchronous control operation, but wait for
+        the result.  See the specific gensios and the C interface for
+        specific gensio controls.
+
+        depth -- The gensio in the stack to choose.  0 selects the top
+            gensio, 1 the second from the top, etc.
+            GENSIO_CONTROL_DEPTH_FIRST will call the control
+            down the stack until a control doesn't return GE_NOTSOP.
+        get -- A boolean specifying if this is a get or a put operation.
+        option -- The specific gensio control to call.
+        data -- A string specifying the data for the control.
+
+        Returns a string with the result data for the control.
+        """
+        return ""
+
     def is_client(self):
         """Return whether the gensio is a client or server."""
         return True
@@ -1274,8 +1362,10 @@ class waiter:
 
         count -- The number of wakeups expected.
         timeout -- The time in milliseconds to wait.
+
+        Returns the number of milliseconds remaining, or 0 if timed out.
         """
-        return
+        return timeout
 
     def wait(self, count):
         """Wait for count wakeups to this object to be called.
