@@ -66,6 +66,11 @@ struct oom_tests {
     char *args;
     bool str_found;
     unsigned int wait_pos;
+    unsigned int state;
+    char strbuf[10];
+    unsigned int strbuf_pos;
+    const char *orig_connecter;
+    int port;
     int err;
     struct gensio_waiter *waiter;
     struct gensio_os_funcs *o;
@@ -81,6 +86,9 @@ static const char *os_func_str = "";
 
 struct gensio_os_proc_data *proc_data;
 
+static int echo_comm_fd = -1;
+static char echo_comm_port[100];
+
 #ifdef _WIN32
 #include <windows.h>
 
@@ -88,7 +96,7 @@ struct gensio_os_proc_data *proc_data;
 typedef int ssize_t;
 #endif
 
-#define DEFAULT_ECHO_COMMPORT "COM1"
+#define DEFAULT_ECHO_COMMPORT ""
 
 bool
 file_is_accessible_dev(const char *filename)
@@ -136,6 +144,7 @@ open_tempfile(char *name, unsigned int len, const char *pattern)
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <sys/ioctl.h>
 #include <unistd.h>
 #include <fcntl.h>
 bool
@@ -239,6 +248,41 @@ get_echo_dev(struct gensio_os_funcs *o, const char *testname,
 	e = te;
     } else if (rv == GE_NOTFOUND) {
 	e = DEFAULT_ECHO_COMMPORT;
+#ifndef _WIN32
+#define SERIALSIM_ALLOC_ID     0x5391
+	if (echo_comm_fd < 0)
+	    rv = open("/dev/ttyEcho", O_RDWR);
+	else
+	    rv = echo_comm_fd;
+	if (rv >= 0) {
+	    echo_comm_fd = rv;
+
+	    rv = ioctl(echo_comm_fd, SERIALSIM_ALLOC_ID, 0);
+	    if (rv >= 0) {
+		int count = 100;
+
+		snprintf(echo_comm_port, sizeof(echo_comm_port),
+			 "/dev/ttyEcho%d", rv);
+		/* Now wait until the device file is available. */
+		do {
+		    rv = open(echo_comm_port, O_RDWR);
+		    if (rv >= 0) {
+			close(rv);
+			e = echo_comm_port;
+			break;
+		    };
+		    count--;
+		    if (count == 0)
+			break;
+		    usleep(10000);
+		} while (true);
+	    }
+	}
+#endif
+	if (e[0] == '\0') {
+	    fprintf(stderr, "echo comm port not set, skipping test.\n");
+	    return false;
+	}
     } else {
 	fprintf(stderr, "Unable to get GENSIO_TEST_ECHO_DEV: %s",
 		gensio_err_to_str(rv));
@@ -276,7 +320,7 @@ static char *ipmisim_config =
     "set_working_mc 0x20\n"
     "\n"
     "  startlan 1\n"
-    "    addr localhost 9001\n"
+    "    addr localhost 0\n"
     "\n"
     "    priv_limit admin\n"
     "\n"
@@ -302,7 +346,10 @@ ipmisim_cb(struct gensio *io, void *user_data, int event, int err,
 	   unsigned char *buf, gensiods *buflen, const char *const *auxdata)
 {
     struct oom_tests *test = user_data;
-    static const char waitstr[4] = "\x0a\x0d> ";
+    static const char waitstr1[17] = "Opened UDP port ";
+    static const char waitstr2[2] = "\x0a";
+    static const char waitstr3[5] = "\x0a\x0d> ";
+    const char *str;
     gensiods pos;
 
     if (event != GENSIO_EVENT_READ)
@@ -333,19 +380,35 @@ ipmisim_cb(struct gensio *io, void *user_data, int event, int err,
     if (test->str_found)
 	return 0;
 
+    if (test->state == 0)
+	str = waitstr1;
+    else if (test->state == 1)
+	str = waitstr2;
+    else if (test->state == 2)
+	str = waitstr3;
+
     for (pos = 0; pos < *buflen; pos++) {
-	if (buf[pos] == (unsigned char) waitstr[test->wait_pos]) {
+	if (buf[pos] == (unsigned char) str[test->wait_pos]) {
 	    test->wait_pos++;
-	    if (test->wait_pos >= sizeof(waitstr)) {
+	    if (test->wait_pos >= strlen(str)) {
 		test->wait_pos = 0;
-		test->str_found = true;
-		gensio_os_funcs_wake(test->o, test->waiter);
+		if (test->state == 2) {
+		    test->str_found = true;
+		    gensio_os_funcs_wake(test->o, test->waiter);
+		    test->strbuf[test->strbuf_pos] = '\0';
+		} else {
+		    test->state++;
+		}
 		break;
 	    }
 	} else {
 	    test->wait_pos = 0;
 	}
+	if (test->state == 1 && test->strbuf_pos < sizeof(test->strbuf) - 1) {
+	    test->strbuf[test->strbuf_pos++] = buf[pos];
+	}
     }
+    *buflen = pos;
 
     return 0;
 }
@@ -394,6 +457,8 @@ ipmisim_start(struct gensio_os_funcs *o, struct oom_tests *test)
     test->err = 0;
     test->str_found = false;
     test->wait_pos = 0;
+    test->state = 0;
+    test->strbuf_pos = 0;
 
     err = str_to_gensio(test->args, o, ipmisim_cb, test, &test->io);
     if (err) {
@@ -440,6 +505,24 @@ ipmisim_start(struct gensio_os_funcs *o, struct oom_tests *test)
 	goto out_err;
     }
 
+    test->port = strtoul(test->strbuf, NULL, 0);
+
+    if (test->orig_connecter) {
+	if (test->connecter)
+	    gensio_os_funcs_zfree(o, test->connecter);
+	test->connecter = NULL;
+    } else {
+	test->orig_connecter = test->connecter;
+	test->connecter = NULL;
+    }
+    test->connecter = gensio_alloc_sprintf(o, test->orig_connecter,
+					   test->port);
+    if (!test->connecter) {
+	printf("Unable to allocate connecter info,\n"
+	       "skipping ipmisol test\n");
+	goto out_err;
+    }
+
     return 0;
 
  out_err:
@@ -624,8 +707,15 @@ check_oom_test_present(struct gensio_os_funcs *o, struct oom_tests *test)
     return test->check_value;
 }
 
+#if HAVE_PTY && !defined(_WIN32)
+# define HAVE_CON_PTY 1
+#else
+/* ptys are not connectable to on windows like on *nix. */
+# define HAVE_CON_PTY 0
+#endif
+
 struct oom_tests oom_tests[] = {
-    { "ipmisol,lan -U ipmiusr -P test -p 9001 localhost,115200", NULL,
+    { "ipmisol,lan -U ipmiusr -P test -p %d localhost,115200", NULL,
       /* In this test some errors will not result in a failure. */
       .allow_no_err_on_trig = true,
       .check_value = HAVE_OPENIPMI,
@@ -670,7 +760,8 @@ struct oom_tests oom_tests[] = {
     },
     { "udp,ipv4,localhost,", "udp,ipv4,0",
       /* In this tests some errors will not result in a failure. */
-      .allow_no_err_on_trig = true
+      .allow_no_err_on_trig = true,
+      .max_io_size = 2000
     },
     { "mux,sctp,localhost,", "mux,sctp,0",
       .check_if_present = check_sctp_present, .check_value = HAVE_LIBSCTP,
@@ -720,7 +811,7 @@ struct oom_tests oom_tests[] = {
        *    It can still be run directly with the -t option.
        */
       .no_default_run = true,
-      .check_value = HAVE_PTY },
+      .check_done = 1, .check_value = HAVE_CON_PTY },
     { "ax25(laddr=test-2,addr=\"0,test-1,test-2\"),kiss(writebuf=512,readbuf=512),tcp,localhost,",
       "ax25(laddr=test-1),kiss(writebuf=512,readbuf=512),tcp,0",
       /* In this tests some errors will not result in a failure. */
@@ -2251,7 +2342,7 @@ main(int argc, char *argv[])
 	rv = gensio_tcl_funcs_alloc(&o);
 #endif
     } else {
-	rv = gensio_default_os_hnd(GENSIO_DEF_WAKE_SIG, &o);
+	rv = gensio_alloc_os_funcs(GENSIO_DEF_WAKE_SIG, &o, 0);
     }
     if (rv) {
 	fprintf(stderr, "Could not allocate OS handler: %s\n",
