@@ -9,7 +9,7 @@ from utils import *
 import gensio
 
 print("Test accept ssl-tcp")
-ta = TestAccept(o, "ssl(CA=%s/CA.pem),tcp,ipv4,localhost," % keydir, "ssl(key=%s/key.pem,cert=%s/cert.pem),tcp,ipv4,0" % (keydir, keydir), do_test, do_close = False,
+ta = TestAccept(o, "ssl(CA=%s/CA.pem),tcp,ipv4,localhost," % keydir, "ssl(key=%s/key.pem,cert=%s/cert.pem),tcp,ipv4,localhost,0" % (keydir, keydir), do_test, do_close = False,
                 expected_raddr = "ipv4,127.0.0.1,")
 print("Get peer cert");
 cn = ta.io1.control(0, gensio.GENSIO_CONTROL_GET,
