@@ -15,7 +15,7 @@ def do_ratelimit_test1(io1, io2, timeout=2000):
     if (io1.handler.wait_timeout(1000) != 0):
         raise Exception(("%s: %s: " % ("do_ratelimit_test", io1.handler.name)) +
                         ("Sent data too fast"))
-    if (io1.handler.wait_timeout(1000) == 0):
+    if (io1.handler.wait_timeout(2500) == 0):
         raise Exception(("%s: %s: " % ("do_ratelimit_test", io1.handler.name)) +
                         ("Sent data too slowly"))
     if (io2.handler.wait_timeout(1000) == 0):
@@ -26,10 +26,10 @@ def do_ratelimit_test2(io1, io2, timeout=2000):
     do_ratelimit_test1(io2, io1, timeout)
 
 print("Test ratelimit gensio")
-TestAccept(o, "ratelimit(xmit_delay=100m),tcp,localhost,", "tcp,0",
+TestAccept(o, "ratelimit(xmit_delay=100m),tcp,localhost,", "tcp,localhost,0",
            do_ratelimit_test1, chunksize = 64)
 print("Test ratelimit accepter")
-TestAccept(o, "tcp,localhost,", "ratelimit(xmit_delay=100m),tcp,0",
+TestAccept(o, "tcp,localhost,", "ratelimit(xmit_delay=100m),tcp,localhost,0",
            do_ratelimit_test2, chunksize = 64)
 del o
 test_shutdown()
