@@ -23,7 +23,7 @@
 #define GENSIOSIG SIGUSR1
 #endif
 
-static struct gensio_os_funcs *o;
+static struct gensio_os_funcs *osh;
 static int global_err;
 static unsigned int channels = 1;
 static unsigned char *playbuf;
@@ -100,7 +100,7 @@ io_event(struct gensio *io, void *user_data, int event, int err,
 	fprintf(stderr, "Error from io: %s\n", gensio_err_to_str(err));
 	global_err = err;
     }
-    gensio_os_funcs_wake(o, waiter);
+    gensio_os_funcs_wake(osh, waiter);
     return 0;
 }
 
@@ -124,7 +124,7 @@ list_sound_devs(struct gensio_os_funcs *o, const char *devtype)
     err = str_to_gensio(gstr, o, NULL, NULL, &lg);
     free(gstr);
     if (err) {
-	fprintf(stderr, "Unable to allocated gensio: %s\n",
+	fprintf(stderr, "Unable to allocate gensio: %s\n",
 		gensio_err_to_str(err));
 	return 1;
     }
@@ -168,7 +168,7 @@ term_handler(void *cb_data)
     struct gensio_waiter *waiter = cb_data;
 
     fflush(stdout);
-    gensio_os_funcs_wake(o, waiter);
+    gensio_os_funcs_wake(osh, waiter);
 }
 
 static const char *progname;
@@ -186,7 +186,8 @@ help(int err)
     printf("  -c, --channels <n> - The number of channels, defaults to 1\n");
     printf("  -s, --bufsize <n> - The buffer size, defaults to 2048\n");
     printf("  -t, --type <n> - The interface type, either alsa (Linux),\n"
-	   "    win (Windows), or file.  Default to alsa or win.\n");
+	   "    win (Windows), portaudio (generally MacOS), or file.  Default\n"
+	   "    to alsa, win, or portaudio depending on the platform.\n");
     printf("  -f, --format <str> - The I/O format.  Default to float.  May\n"
 	   "    be one of double, float, int32, int24, int16, or int8\n");
     printf("  -p, --play - Playback data from stdin.  The default is to\n"
@@ -213,6 +214,7 @@ main(int argc, char *argv[])
     bool play = false;
     const char *devtype = NULL; /* Take the default by default. */
     const char *format = "float";
+    struct gensio_os_funcs *o;
 
     progname = argv[0];
 
@@ -255,14 +257,18 @@ main(int argc, char *argv[])
 		    argv[arg]);
 	    return 1;
 	}
+	if (rv < 0)
+	    return 1;
     }
 
-    rv = gensio_default_os_hnd(GENSIOSIG, &o);
+    rv = gensio_alloc_os_funcs(GENSIOSIG, &osh, 0);
     if (rv) {
 	fprintf(stderr, "Could not allocate OS handler: %s\n",
 		gensio_err_to_str(rv));
 	return 1;
     }
+    /* osh is global, get a local copy for convenience. */
+    o = osh;
     gensio_os_funcs_set_vlog(o, do_vlog);
 
     rv = gensio_os_proc_setup(o, &proc_data);
