@@ -81,6 +81,7 @@ static bool permit_root = false;
 static bool pw_login = false;
 static bool do_2fa = false;
 static bool ginteractive_login = true;
+static bool use_login = USE_LOGIN_PROGRAM;
 
 #ifndef _WIN32
 /*
@@ -88,7 +89,8 @@ static bool ginteractive_login = true;
  * PAM.  This way we can do 2-factor auth with certificates.
  */
 static const char *pam_cert_auth_progname;
-static const char *pid_file = NULL;
+static const char *pid_file;
+static const char *pam_service;
 
 /* We fork in *nix. */
 #define DO_FORK 1
@@ -97,9 +99,9 @@ static const char *pid_file = NULL;
 #define log_event syslog
 #define vlog_event vsyslog
 static void
-start_log(bool debug)
+start_log(bool idebug)
 {
-    if (!debug)
+    if (!idebug)
 	openlog(progname, 0, LOG_AUTH);
     else
 	openlog(progname, LOG_PID | LOG_CONS | LOG_PERROR, LOG_DAEMON);
@@ -303,13 +305,6 @@ struct per_con_info {
     unsigned int oobpos;
 };
 
-static const char *default_keyfile =
-    SYSCONFDIR DIRSEPS "gtlssh" DIRSEPS "gtlsshd.key";
-static const char *default_certfile =
-    SYSCONFDIR DIRSEPS "gtlssh" DIRSEPS "gtlsshd.crt";
-static const char *default_configfile =
-    SYSCONFDIR DIRSEPS "gtlssh" DIRSEPS "gtlsshd.conf";
-
 static int
 write_s_nl_addc(struct gensio *io, char *obuf, char c,
 		gensiods *pos, gensiods len, gensio_time *timeout)
@@ -493,7 +488,7 @@ auth_free(struct auth_data *auth)
     if (auth->locport)
 	free_local_ports(auth->locport);
     if (auth->homedir)
-	o->free(o, auth->homedir);
+	free(auth->homedir);
     if (auth->ushell)
 	o->free(o, auth->ushell);
     if (auth->passwd) {
@@ -1082,7 +1077,9 @@ setup_auth(struct auth_data *auth)
     const char *pn;
 
     pn = progname;
-    if (pam_cert_auth_progname && auth->authed_by_cert)
+    if (pam_service)
+	pn = pam_service;
+    else if (pam_cert_auth_progname && auth->authed_by_cert)
 	pn = pam_cert_auth_progname;
     auth->pam_conv.appdata_ptr = auth;
     auth->pam_err = pam_start(pn, auth->username, &auth->pam_conv, &auth->pamh);
@@ -1368,6 +1365,10 @@ update_privileges(HANDLE h, struct priv_data *privs, unsigned int privs_len)
 	malloc(sizeof(TOKEN_PRIVILEGES) +
 	       sizeof(LUID_AND_ATTRIBUTES) * (hpriv->PrivilegeCount +
 					      privs_len));
+    if (!nhpriv) {
+	err = STATUS_NO_MEMORY;
+	goto out_err;
+    }
     nhpriv->PrivilegeCount = hpriv->PrivilegeCount;
 
     for (j = 0; j < privs_len; j++)
@@ -1396,8 +1397,7 @@ update_privileges(HANDLE h, struct priv_data *privs, unsigned int privs_len)
     }
     err = 0;
  out_err:
-    if (hpriv)
-	free(hpriv);
+    free(hpriv);
     if (nhpriv)
 	free(nhpriv);
     return err;
@@ -2208,9 +2208,12 @@ new_rem_io(struct gensio *io, struct auth_data *auth)
 	if (err)
 	    goto out_bad_vals;
 #ifdef _WIN32
-	s = gensio_alloc_sprintf(o, "pty,%s -i", auth->ushell);
+	s = gensio_alloc_sprintf(o, "pty,%s", auth->ushell);
 #else
-	s = gensio_alloc_sprintf(o, "pty,-%s -i", auth->ushell);
+	if (use_login)
+	    s = gensio_alloc_sprintf(o, "pty,login -f %s", auth->username);
+	else
+	    s = gensio_alloc_sprintf(o, "pty,-%s -i", auth->ushell);
 #endif
 	login = true;
 	do_chdir = true;
@@ -2746,7 +2749,10 @@ handle_new(struct gensio *net_io)
     if (top_io)
 	gensio_free(top_io);
     auth_free(auth);
+#ifdef _WIN32
     if (oneshot)
+#endif
+    /* Terminate the subprocess. */
 	gensio_os_funcs_wake(o, ginfo->waiter);
 }
 
@@ -2992,9 +2998,47 @@ acc_event(struct gensio_accepter *accepter, void *user_data,
     return 0;
 }
 
+const char *keyfile = NULL;
+const char *certfile = NULL;
+
+static int
+setup_keyfiles(void)
+{
+    const char *confdir = NULL;
+
+    if (keyfile && certfile)
+	return 1;
+
+    confdir = get_confdir();
+    if (!confdir) {
+	log_event(LOG_ERR, "Unable to get gtlssh config dir\n");
+	return 0;
+    }
+    if (!keyfile) {
+	keyfile = alloc_sprintf("%s%sgtlsshd.key", confdir, DIRSEPS);
+	if (!keyfile) {
+	    log_event(LOG_ERR, "Unable to allocate key file\n");
+	    return 0;
+	}
+    }
+
+    if (!certfile) {
+	certfile = alloc_sprintf("%s%sgtlsshd.crt", confdir, DIRSEPS);
+	if (!certfile) {
+	    log_event(LOG_ERR, "Unable to allocate cert file\n");
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
 static void
 help(int err)
 {
+    if (!setup_keyfiles())
+	exit(1);
+
     printf("%s [options]\n", progname);
     printf("\nA program to connect gensios together.  This programs has two\n");
     printf("gensios, io1 (default is local terminal) and io2 (must be set).\n");
@@ -3004,7 +3048,9 @@ help(int err)
     printf("  -d, --debug - Enable debug.  Specify more than once to increase\n"
 	   "    the debug level\n");
     printf("  -c, --certfile <file> - The certificate file to use.\n");
+    printf("      Default is %s\n", certfile);
     printf("  -h, --keyfile <file> - The private key file to use.\n");
+    printf("      Default is %s\n", keyfile);
     printf("  --permit-root - Allow root logins.\n");
     printf("  --allow-password - Allow password-based logins.\n");
     printf("  --oneshot - Do not fork new connections, do one and exit.\n");
@@ -3021,6 +3067,12 @@ help(int err)
     printf("  --pam-cert-auth <name> - When doing a certificate auth,\n");
     printf("     use the name as the PAM program name and run the PAM auth\n");
     printf("     after the certificate auth succeeds.  For 2-factor auth.\n");
+    printf("  --pam-service <name> - Use the given name for the PAM,\n");
+    printf("     authentication service.  Default is the program name.\n");
+    printf("  --use-login, --no-use-login - Use or don't use the login\n");
+    printf("     program for interactive logins.  The default is to\n");
+    printf("     %s login.\n", USE_LOGIN_PROGRAM ? "use" : "not use");
+    printf("  --no-use-login - Don't use login program.\n");
     printf("  -P, --pidfile <file> - Create the given pidfile.\n");
 #endif
     printf("  --version - Print the version number and exit.\n");
@@ -3068,9 +3120,6 @@ main(int argc, char *argv[])
     int arg, rv;
     struct gensio_os_funcs *o;
     struct gdata ginfo;
-    const char *keyfile = default_keyfile;
-    const char *certfile = default_certfile;
-    const char *configfile = default_configfile;
     unsigned int port = 852;
     char *s;
     bool notcp = false, sctp = false;
@@ -3094,9 +3143,6 @@ main(int argc, char *argv[])
 	if ((rv = cmparg_uint(argc, argv, &arg, "-p", "--port",
 			      &port)))
 	    ;
-	else if ((rv = cmparg(argc, argv, &arg, "-f", "--configfile",
-			      &configfile)))
-	    ;
 	else if ((rv = cmparg(argc, argv, &arg, "-c", "--certfile",
 			      &certfile)))
 	    ;
@@ -3120,6 +3166,9 @@ main(int argc, char *argv[])
 	else if ((rv = cmparg(argc, argv, &arg, NULL, "--pam-cert-auth",
 			      &pam_cert_auth_progname)))
 	    ;
+	else if ((rv = cmparg(argc, argv, &arg, NULL, "--pam-service",
+			      &pam_service)))
+	    ;
 	else if ((rv = cmparg(argc, argv, &arg, "-P", "--pidfile", &pid_file)))
 	    ;
 #endif
@@ -3129,6 +3178,10 @@ main(int argc, char *argv[])
 	    daemonize = false;
 	else if ((rv = cmparg(argc, argv, &arg, NULL, "--nointeractive", NULL)))
 	    ginteractive_login = false;
+	else if ((rv = cmparg(argc, argv, &arg, NULL, "--use-login", NULL)))
+	    use_login = 1;
+	else if ((rv = cmparg(argc, argv, &arg, NULL, "--no-use-login", NULL)))
+	    use_login = 0;
 	else if ((rv = cmparg(argc, argv, &arg, "-4", NULL, NULL)))
 	    iptype = "ipv4,0.0.0.0,";
 	else if ((rv = cmparg(argc, argv, &arg, "-6", NULL, NULL)))
@@ -3155,9 +3208,12 @@ main(int argc, char *argv[])
 
     if (!sctp && notcp) {
 	log_event(LOG_ERR, "You cannot disable both TCP and SCTP\n");
-	exit(1);
+	return 1;
     }
 
+    if (!setup_keyfiles())
+	return 1;
+
     if (checkout_file(glogger, NULL, keyfile, false, true))
 	return 1;
     if (checkout_file(glogger, NULL, certfile, false, false))
@@ -3166,7 +3222,7 @@ main(int argc, char *argv[])
     memset(&ginfo, 0, sizeof(ginfo));
     gensio_list_init(&ginfo.auths);
 
-    rv = gensio_default_os_hnd(0, &o);
+    rv = gensio_alloc_os_funcs(0, &o, 0);
     if (rv) {
 	log_event(LOG_ERR, "Could not allocate OS handler: %s\n",
 		  gensio_err_to_str(rv));
